#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# search.cgi
#    search site script
#    990828 RF
#
####
 
####
#    
# Version 0.8.34 TG 20000524
#    fixed problem with language support 
#       and new cell template names
# Version 0.8.32 GF 20000520
#    retrofit for 0.9 datastructure
# Version 0.8.31 JC 20000418
#    replaced loadtemplate with localize_template
#    included permute_include
#    removed $skeleton
# Version 0.8.30 IB 20000411
#    default to language in site.pm
# Version 0.8.29 IB 20000410
#    database changes
# Version 0.8.28 IB 20000410
#    multilingual messages
# Version 0.8.27 IB 20000402
#    topbarnav -> jump_to_nav
# Version 0.8.26 IB 20000401
#    new directory structure
# Version 0.8.25 IB 20000330
#    changed connect statement
# Version 0.8.24 JC 20000329
#    changed DBI prepare/connect
# Version 0.8.23 IB 20000327
#    corrected $relpath
# Version 0.8.22 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.8.21 JC 20000323
#    fix for bug #56 on line 280
# Version 0.8.20 IB 20000317
#    new template names
# Version 0.8.19 IB 20000302
#    updated copyright
# Version 0.8.18 IB 20000226
#    change store to site
# Version 0.8.17 IB 20000119
#    limited the search to only displayable items
# Version 0.8.16 IB 20000111
#    corrected search output
# search.cgi 0.8.15 GF 991203
#    search just display items using status="pn" 
# search.cgi 0.8.14 GF 991203
#    added /site to /bits
#    changed page name
#    changed cell names
#    general cleanup
# search.cgi, 0.8 series
#    total rewrite as of 990828
# Version 0.8.1 990828 RF
#    core querry construction engine 
#
###

###
# Overview:
#   Assemble search parameters
#   Obtain list of fields to query against by special term
#   Construct field-specific querries
#   Determine effected tables
#   Run querries by table
#   construct table-by-table result sets
#      NOTE - only tables searched now are:
#         things
#         variants
#         extraprop
#         prices
#         linkage?
#         thesaurus?
#   perform eliminative combinatorics
#   prepare final result set
#   handoff to display routines
#
###

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in cgi-bin/om,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain cgi-bin
if ( $s !~ /cgi-bin/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "cgi-bin" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - cgi-bin is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

use DBI;
require "$relpath/lib/cgi-lib.pm";
require "$relpath/lib/formtools.pm";
require "$relpath/lib/filetools.pm";
require "$relpath/conf/site.pm";
require "$relpath/lib/state.pm";
require "$relpath/lib/localization.pm";

## establish site-specific parameters
#
$site=setsite();

## reading cgi stuff
#
%FORM=();
&ReadParse; 

$om_render_lang=$FORM{'OM_UI_LANGUAGE'};
$om_render_lang=$site->{'om_locale_lang_def'} if ( ! $om_render_lang );
$om_render_lang="eng" if ( ! $om_render_lang );

&setsessionid();

######
### Assemble search parameters
######
#   Syntax for form variables:
#      search_FIELDNAME_inp (value can contain range specifiers)
#      search_parameter_inp (generally lacks range specifiers)
#      
$sinput={};           # empty hashref of search terms
foreach $k ( keys %FORM ) {

   next if ( $k !~ /^search_/ );       # skip if it isn't search
   next if ( $k !~ /_inp$/ );          # skip if it isn't an input
   next if ( ! $FORM{$k} );            # skip blank parameters
   $ki = $k;
   $ki =~ s/^search_// ;               # generate new key   
   $ki =~ s/_inp$// ;                  # trim "input" flag
   $sinput->{$ki}=$FORM{$k};           # create entry in search term hash

} #/done constructing earch term list


######
### Catch the "redirect" entries and act
######
$rtarget="";                            # target for redirection
foreach $k ( keys %{$sinput} ) {
   next if ( $k !~ /^redirect/ );
   $rtarget=$sinput->{$k} if ( $sinput->{$k} );
} # done looking at $sinput->  keys for redirects

$rtarget="" if ( length($rtarget)<8 );  # minimum sku length

if ( $rtarget ) {
   $rtarget="/html/$om_render_lang/$rtarget.shtml" 
   if ( $rtarget !~ /(.shtml$|^\/cgi-bin|^http:)/ );
   print "Status: 302/Moved\n";
   print "Location: $rtarget\n";
   print "\n\n";

   exit 1;
} #

## defaults that don't live elsewhere yet
#
$sku="";
$startnum = 0;
if ( exists($FORM{'startnum'} )) {
   $startnum = $FORM{'startnum'} if ( $FORM{'startnum'} > 0 );
} # done checking on passed startnum

## establish connection to table
#
$s = "$site->{'db_site_config'}"; 
$dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});


######
### Initialize "constants"
######
$maxrec=200;                           # max recs to pull per table
%fieldtype=();                         # field types
%fieldlist=();                         # field lists by table
%prikey=();                            # primary key field by table

## load field types for fields of interest
#     TBD - load from table descriptions
#
$fieldtype{'brief'}="char";  
$fieldtype{'msku'}="char";  
$fieldtype{'title'}="char";  
$fieldtype{'custid'}="char";  
$fieldtype{'u_email'}="char";  
$fieldtype{'uname'}="char";  
$fieldtype{'objid'}="char";  
$fieldtype{'vcode'}="char";  

## load field lists by table
#
$fieldlist{'things'}="objid,msku,vcode,title,brief,axisnames";
$fieldlist{'contacts'}="objid,u_email,uname";
$fieldlist{'addresses'}="name,city,state,postcode";

## load primary keys by table
#
$prikey{'things'}="objid";
$prikey{'extraprop'}="objid";

$big_fieldlist = join ",", ( values %fieldlist );

######
### Obtain mapping of special terms to fields
######
%termmap=();                           # blank term map hash
#                                      # TBD - obtain from database
$termmap{'keywords'}= join ":", ( 
   "title",
   "brief",
   "descript",
   "msku",
   "objid",
   "full_desc"
   );
$termmap{'partno'}= join ":", (
   "msku",
   "objid",
   "isbn",
   "upc"
   );


######
### First Wave of Construct list of querries by field - mapped fields
######
#   precedence: 
#      a precise fieldname excludes broader mappings from the field
#      a field is querried against a table it is a key of

%fieldq=();                          # holder for growing querries by field
%where=();                           # where clauses by table
foreach $k ( keys %{$sinput} ) {     # first wave - mappings
   next if ( $k !~ /[a-z]/ );        # skip all uppercase keys this pass
   next if ( ! $termmap{$k} );       # skip unmapped keys
   next if ( ! $sinput->{$k} );      # skip null inputs
   $sinput->{$k} =~ s/'/''/g;
   foreach $j ( split /:/, $termmap{$k} ) {   # run through mapped fields CHAR?
      $fieldq{$j}.=" OR " if ( $fieldq{$j} );
      #$fieldq{$j}='{'.$fieldq{$j} if ( ( $fieldq{$j} )
      #                 and ( $fieldq{$j} !~ /^\{/ ) );   
      $fieldq{$j}.="( $j like '\%$sinput->{$k}\%' )";
   }                                 # done running through mapped fields
}                                    # done with first wave

                                # Construct "OR"ed table-level
                                # "WHERE" clauses since mapped terms are
                                # "or"ed together. Exact fieldnames are
                                # anded together after second wave

foreach $k ( keys %fieldlist ) {             # run through relevant fields
   foreach $j ( split /\,/, $fieldlist{$k} ) {
      $where{$k}.=" OR " if ( ( $where{$k} ) 
                            and ( $fieldq{$j} ) );
      $where{$k}="{".$where{$k} if ( ( $where{$k} ) 
         and ( $where{$k} !~ /^\{/ ) and ( $fieldq{$j} ) );
      $where{$k}.=$fieldq{$j} if ( $fieldq{$j} );
   }                                         # done running through fields 
}                                            # done examining table


                                # Enclose multiple "OR" WHERE clauses
foreach $k ( keys %where ) {                 # run through tables
   next if ( $where{$k} !~ /^\{/ );          # skip if not start with {
   $where{$k} =~ s/^\{// ;                   # kill leading {
   $where{$k} = "(".$where{$k}.")";          # enclose with ()
}                                            # done running through tables

######
### Second Wave - exact search fields
######
                                # Flush non-extraprop field-level queries
foreach $k ( keys %fieldq ) {   # run through field level querries
   delete $fieldq{$k} 
   if ( $big_fieldlist =~ /(^|,)$k(,|$)/ );  # eliminate handled fields
} # done flushing fields that may occour in both waves

foreach $k ( keys %{$sinput} ) {     # second wave - exact fields
   next if ( $k =~ /[a-z]/ );        # skip keys with lowercase this pass
   $fieldq{$k}.=" AND " if ( $fieldq{$k} );

   ## minimal sensitivity to the field type
   #
   if ( $fieldtype{$k} !~ /char/ ) { # not character data 
      #                              # strip illegal characters
      $sinput->{$k} =~ tr/0123456789.><=//cd;
      #                              # TBD - allow +/- and kill 
      #                              #       internal gt/lt/eq 
      #                              # look for gt/lt/eq operators
      if ( $sinput->{$k} =~ /^(\<\=|\=\<)/ ) {       # less than or equal
         $fieldq{$k}.="( $k<=";
      } elsif ( $sinput->{$k} =~ /(\>\=|\=\>)$/ ) {  # less than or equal
         $fieldq{$k}.="( $k<=";
      } elsif ( $sinput->{$k} =~ /(\<\=|\=\<)$/ ) {  # greater than or equal
         $fieldq{$k}.="( $k>=";
      } elsif ( $sinput->{$k} =~ /^(\>\=|\=\>)/ ) {  # greater than or equal
         $fieldq{$k}.="( $k>=";
      } elsif ( $sinput->{$k} =~ /(^\>|\<$)/ ) {   # greater than
         $fieldq{$k}.="( $k>";
      } elsif ( $sinput->{$k} =~ /(^\<|\>$)/ ) {   # less than
         $fieldq{$k}.="( $k<";
      } else {                                     # equal
         $fieldq{$k}.="( $k=";
      }                                            # done with inequality
      #                                            # remove <>=
      $sinput->{$k} =~ tr/<>=//d ;
      $fieldq{$k}.="$sinput->{$k} ) ";
      $fieldq{$k}="" if ( ! $sinput->{$k} );       # just in case nothing left
   } else {                                  # assume character of some sort
      $fieldq{$k}.="( $k like '\%$sinput->{$k}\%' )";
   }                                         # /character or numeric decision
}                                            # done with second wave

foreach $k ( keys %fieldq ) {                # check for compound clauses
   next if ( $fieldq{$k} !~ /^\{/ );         # looking for initial curly brace
   $fieldq{$k} =~ s/^\{// ;                  # remove leading curly brace
   $fieldq{$k}="($fieldq{$k})";              # enclose the group   
}                                            # done checking compound clauses

@mapped=();                                  # list of covered fields
foreach $k ( keys %fieldlist ) {             # run through relevant fields
   foreach $j ( split /\,/, $fieldlist{$k} ) {
      $where{$k}.=" AND " if ( ( $where{$k} ) 
                            and ( $fieldq{$j} ) );
      $where{$k}.=$fieldq{$j} if ( $fieldq{$j} );
   }                                         # done running through fields 
}                                            # done examining table

######
### Construct Exraprop Querry (any "none of the above" fields)
######
#                               # EXTRAPROP search
                                # FIELDNAME,BODY searched, OBJID relevant
$use_objid="";
$use_objid=$sinput->{'objid'} if ( $sinput->{'objid'} );

foreach $k ( keys %fieldq ) {   # run through field level querries
   next if ( $big_fieldlist =~ /(^|,)$k(,|$)/ );  # eliminate handled fields
   #next if ( $fieldq{$k} =~ /[<>=]/ ) ;           # numeric? handle elsewhere  
   next if ( ! $fieldq{$k} );                     # has value?
   $where{'extraprop'}.=" OR " if ( ( $where{'extraprop'} ) 
       and ( $fieldq{$k} ) );
   # parse the field query into more complex format
   $s=$fieldq{$k};
   if ( $use_objid ) {          # use objectid in query
     $u="( OBJID LIKE '\%$use_objid\%' AND FIELDNAME='$k' AND BODY";
     $s=~s/^\( $k/$u/e;
   } else {                     # do not use objectid in query
     $u="( FIELDNAME='$k' AND BODY";
     $s=~s/^\( $k/$u/e;
   }                            # done with field translation
   $where{'extraprop'} .= $s;
}                               # done with construction of extraprop

###### 
### Kill off uninformative or degenerate querries
######

######
### Execute Querries
######
%rows=();                                    # returned search rows
foreach $k ( keys %where ) {
   $q="SELECT \* from $k where $where{$k}";
   $sth=$dbh->prepare( $q );
   $sth->execute;
   $num=$sth->rows;
   $numr=0;                                  # count of returned recs
   $result={};                               # blank hashref
   while (( $numr < $maxrec ) and ( $numr < $num )) {
      $h=$sth->fetchrow_hashref;
      $i=$numr;                              # default handle
      if ( $prikey{$k} ) {
         $j=$prikey{$k};                     # get key to use for table
         if ( $j eq "objid" ) {

            $i=$h->{'objid'};                 # construct psudofield VSKU
            $h->{'objid'}=$i;                 # populate field

         } else {
            $i=$h->{$j};                     # use key
         }
      }                                      # done with key entry
      $result->{$i}=$h;                      # add record to hashref
      $numr++;
   }                                         # done grabbing results
   
   $rows{$k} = $result if ( $num > 0 );      # only make key if >0
}                                            # done running searches

######
### Construct Result Set
######
#            Strategy:
#
#               all querries connect on SKU/OBJID
#               go through and remove "identifiable" bad records
#               add records to search based on other table results
#
######
### Expansive pass, pull "things" records for other table matches
######
%results=();
foreach $k ( keys %rows ) {
   next if ( $k eq "things" );                # flesh out those records next
   ## check and add things records as needed
   foreach $h ( keys %{$rows{$k}} ) {         # go through records  
      $sku=$h->{$prikey{$k}};                  
      if ( ! $rows{'things'}->{$sku} ) {      # load the things record    
         $q="SELECT \* from things where objid='$sku'"; 
         $sth = $dbh->prepare( $q );
         $sth->execute;
         $num=$sth->rows;
         foreach ( 1..$num ) {
            $rec = $sth->fetchrow_hashref;
            $rec->{'VSKU'}=$h->{$prikey{$k}};
            $rows{'things'}->{$rec->{'SKU'}}=$rec;
         }  # /instance                       # done loading results 
      }     # /sku                            # end of loading this one
   }        # /table record                                       
}           # /table


###### 
### sort records by sort key and cut set to necessary entries
######
#        # $startatrow - lowest number result, first is zero
#        # $resultsperpg - max results per page
#        # $sortfield - name of field to sort results by
#
$sortfield='title';                           # HACK, absract later
$startatrow=0 if ( ! $startatrow );           # default start at first rec
$resultsperpg=12 if ( ! $resultsperpg );      # default 12 per pg.
@sort = ();    # blank array for sort results
foreach $k ( keys %{$rows{'things'}} ) {
    push @sort, $rows{'things'}->{$k};
} # done constructing array


## actually sort the stuff now
#
@sort = sort { lc($a->{$sortfield}) cmp lc($b->{$sortfield}) } @sort;

$resultcount= $#sort + 1;
$resultcount=0 if ($resultcount < 0);

$lastrec= $startnum + $resultsperpg - 1 ;

if ( $resultcount > $lastrec ) {$nextstartnum=$startnum+$resultsperpg; $nextflag="yes";} 

$lastrec=$resultcount if ( $lastrec > $resultcount );
@sort = @sort[$startnum..$lastrec];
$resultcount++ if ( $resultcount>0 );


######
### add generation cycle necessary records to $rows{'things'}
######
$q = "";          
foreach $k ( @sort ) {
#   $b = $rows{'things'}->{$k}->{'SKU'};  
   $b = $rows{'things'}->{$k}->{'objid'};  
   $q.=" OR objid='$b' " if (( $q ) and ( $q !~ /$b/ )
      and ( ! $rows{'things'}->{$b} ));
   $q =" objid='$b' " if (( ! $q ) and ( $q !~ /$b/ )
      and ( ! $rows{'things'}->{$b} ));
}


######
### flesh out all records in $rows{'things'}
######
$q = "";          
foreach $k ( keys %{$rows{'things'}} ) {
   $b = $rows{'things'}->{$k}->{'objid'};  
   $q.=" OR objid='$b' " if (( $q ) and ( $q !~ /$b/ )
      and ( ! $rows{'things'}->{$b} ));
   $q =" objid='$b' " if (( ! $q ) and ( $q !~ /$b/ )
      and ( ! $rows{'things'}->{$b} ));
}


## grab extra records for things


## run query against extraprop
#

$q2 = "SELECT \* from extraprop where $q ";

######
### Construct output page
######

print "Content-type: text/html\n\n";

$cell_np=localize_template("cell_search_intermediate",$om_render_lang,"html",0);
$cell_pr=localize_template("cell_search_product",$om_render_lang,"html",0);
$thispage=localize_template("page_search_result",$om_render_lang,"html",0);

`$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/cgi-bin/om/track.cgi`;

if ($resultsperpg > $resultcount) {$breaktot=$resultcount;} else {$breaktot=$resultsperpg};
$breakat1=int($breaktot/2);
$breakat2=$breakat1+$breakat1;


if ($breakat1 > $#sort) {
	$breakat1=$#sort;
}
if ($breakat2 > $#sort) {
        $breakat2=$#sort;
}

if ($startnum > 0) {$prevstartnum=$startnum-$resultsperpg; $prevflag="yes";}

 for ($i=0; $i <= $#sort; $i++) {
 

   next if ($#sort == 0);	

   $sku=$sort[$i]->{'objid'};

    next if (!$sku);
#   if ($sku =~ /xscm/) {$resultcount--;}
#   next if ($sku =~ /xscm/);
   
   if ($sku =~ /^x/ ) { # non-sellable sku
       $u=$cell_np;
   } else {
       $u=$cell_pr;
   }
   $h=$sort[$i];

    $q=permutebit($u,$h);
   if ($i < $breakat1) {
      $expand1.=$q;
   } elsif ($i < $breakat2 && $i >= $breakat1) {
       $expand2.=$q;
   }

}
$h={};
$h->{'headergif'}="search_results.gif";
$h->{'sitename'}=$site->{'sitename'};
$h->{'title'}="Search Results!";
$h->{'column_220_1'}=$expand1;
$h->{'column_220_2'}=$expand2;
$h->{'brief'}="Your search returned ";

if ($resultcount) {
  $h->{'brief'}.=" these matches! ";
} else {
    $h->{'brief'}.="nothing! Please try again!<P>";
}
$h->{'site_background'}=$site->{'site_background'};

$h->{'os_render_language'}=$om_render_lang;
$h->{'os_language'}=$om_render_lang;

## output the file to stdout
#

$thispage = permutebit( $thispage, $h);
$thispage = permutebit( $thispage, \%FORM );
$thispage = permutebit( $thispage, $site );
$thispage = permutebit( $thispage, \%ENV );
$thispage = permuteclean ( $thispage  );
$thispage = permute_include( $thispage, $om_render_lang, "html", 0); 

print $thispage;

1;




