#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# generate_temps.pl
#
# Program to generate temporary data files
#    to feed into various general pulldowns
# 
# Version 0.9.3 RF 20000523
#    refitted for new data structure and classes
#    note - this is not generalized properly, but to work with
#       the expectations of the various administrative scripts
#       prior to their pending deprecation in favor of admin_obj 
# Version 0.9.2 IB 20000502
#    corrected the sequence of statements
# Version 0.9.1 IB 20000426
#    introduced $script variable
# Version 0.7.15 JC 20000412
#    removed AUTHOR
# Version 0.7.14 IB 20000411
#    $def_lang -> $om_render_lang
# Version 0.7.13 JC 20000411
#    default to eng if no default lang
# Version 0.7.12 JC 20000410
#    multilingual support
# Version 0.7.11 IB 20000402
#    new directory structure
# Version 0.7.10 JC 2000330
#    changed connect statement
# Version 0.7.9 JC 20000328
#    changed DBI prepare/connect
# Version 0.7.8 IB 20000327
#    corrected $relpath
# Version 0.7.7 IB 20000323
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.6 IB 20000301
#    updated copyright
# Version 0.7.5 IB 20000229
#    changed table to database
# Version 0.7.4 IB 20000223
#    changed the name store.pm to site.pm
# Version 0.7.3 IB 991217
#    eliminated ERLOG
# Version 0.7.2 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    calculation of a relative path
#    using of new global variables
# Version 19 RF 990908
#    relative path symlinks - removed absolute symlink paths
#    changed libraries to relative paths
#    added test for handler-less sku
#
# Version 12 SH 990707
#    moved to system 6 data structure and queries
#    if encoding does not have relevant field, sku.sel is created
# Version 11 GF 990310
#    fixed problem regarding nobody running the script
# Version 10 RF 981219
#    removed refs to $datahost
#    removed all calls to machine.pm
# Version 9 RF 981127
#    fixed generation of pulldowns
# Version 8 RF 981124
#    added fieldname binding detection, generates fieldname.sel
#    files as appropriate in addition to XEC sku filenames
# Version 7 RF 981123
#    takes a storecode as an argument
# Version 6 RF 981122
#    multi-store capacity and bits/sel useage
# Version 5 RF 981111
#    converted to DBI interface
# Version 4 RF 981103 
#    added sku.sel generation for encodings
# Version 3 RF 981103 lost
# Version 2 RF 981013
#    added string conversion from mysql to text
# Version 1 RF 981012
#

## determine path from store root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store (site) root

$relpath .= "/../$siteshortname";

use DBI;

require "$relpath/lib/formtools.pm";
require "$relpath/conf/site.pm";

## establish site-specific parameters
#
$site=setsite();
$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );

$verbose_errors=1;
%script_err=();
$script=( split /\./, $scriptname )[0];
&message;

## establish connection in
$s = "$site->{'db_site_config'}"; 
$dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{' site_dbipwd'}); 

$datapath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$om_render_lang/site/sel";

chdir( $datapath );          # move into the datapath directory

#
## generate brands pulldown data file
#
open(FH, ">$datapath/BRAND.sel") || die "$script_err{1} $datapath/BRAND.sel";
# this should be a loop for more than just merchandise...
$q="select objid,title from things where objclass='135-A' ";
$q.=" and ( status='15000-A' ) ";
$q.=" order by title ";
$sth=$dbh->prepare($q);
$sth->execute;
$num=$sth->rows;
for ($x=0; $x<$num; $x++ ) {
   ($k,$f,$v) = $sth->fetchrow_array;
   $v=string_exmysql($v);
   $f.=$sitecode;
   print FH "$k\|$f $v \n";
}
close(FH);

#
## generate encoding-specific pulldown .sel files
#
$q="select objid,body from extraprop where objid='$ARGV[1]' and fieldname='descript'";
$sth=$dbh->prepare($q);
$sth->execute;
$num=$sth->rows;


for ($x=0; $x<$num; $x++ ) {
   ($k,$v) = $sth->fetchrow_array;
   $target="$k".".sel";
   open(FH, ">$datapath/$target") || die "Unable to open $datapath/$target";
   print FH " \|- Not Specified\n";
   $v =~ s/\015/\n/sg ;
   $v =~ s/\012/\n/sg ;
   $v=string_exmysql($v);
   ## replace first colon with vertical slash
   #
   $v =~ s/(^\S+):/$1\|/mg ;
   $v =~ s/^\s+//mg ;


   ## HACK to get rid of blank lines in the string...
   @temp = split /\n/, $v;
   @temp2=();
   while (@temp) {
      $k = shift @temp;
      push @temp2,$k  if ( $k );
   }
   $v = join "\n", @temp2;
   print FH "$v";
   close(FH);
}

#
## copy the encoding-specific pulldown files as FIELDNAME.sel
#
$q="select objid,handler from things where objid='$ARGV[1]' ";
$sth = $dbh->prepare($q);
$sth->execute;
$num=$sth->rows;
for ( $x=0; $x<$num; $x++ ) {
   ($k,$v) = $sth->fetchrow_array;
   if (!$v) { $v=$k };                # if no handler, use sku as handler
   $v=string_exmysql($v);            # graceful handling of underscore
  
   # these things should be lowercase if field names
   $target=$v.".sel";
   $source=$k.".sel"; 
   if ( -e "$datapath/$source" ) {  # source exists
      `/bin/ln -s --f $source $target` if ( $target ne $source );
   } # end of testing for sourcefile existence
} # done running through encoding skus


$sth->finish;
$dbh->disconnect if ( $dbh );

`chown nobody:nobody $datapath/*.sel`;

sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below
   if ( $verbose_errors ) {
      if ($om_render_lang eq "eng" ) {
         $script_err{1}="Unable to open";
      } else {
         open( EM, "<$relpath/bits/$om_render_lang/system/$script.errors");
         while(<EM>) {
            chomp;
            next if /^\#/;
            next if (! $_);
            @temp=split /\t/;
            $script_msg{$temp[0]}=$temp[1];
         }
         close(EM);
      }
   }
} 

1;
