#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# run_stats.pl
#
# Create daily summary report on core statistics
#
# Version 0.7.21 IB 20000606
#    deleted redundant comment lines
# Version 0.7.20 IB 20000603
#    further conversion
# Version 0.7.19 IB 20000602
#    converted to Rel. 1.0.1
# Version 0.7.18 IB 20000531
#    added custom-lib.pm
# Version 0.7.17 IB 20000411
#    $def_lang -> $om_render_lang
# Version 0.7.16 JC 20000411
#    default to eng if no default lang
# Version 0.7.15 JC 20000330
#    changed connect statement
# Version 0.7.14 JC 20000328
#    changed DBI prepare/connect
# Version 0.7.13 IB 20000327
#    corrected $relpath
# Version 0.7.12 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.11 IB 20000316
#    new directory structure
# Version 0.7.10 IB 20000302
#    removed -w
# Version 0.7.9 IB 20000301
#    updated copyright
# Version 0.7.8 IB 20000229
#    changed table to database
# Version 0.7.7 IB 20000223
#    changed the name store.pm to site.pm
# Version 0.7.6 JZ 20000215
#    display actual checked out orders, use templates
# Version 0.7.5 JZ 20000207
#    corrected the numbers
# Version 0.7.4 IB 20000204
#    changed "Snapshot view" to "Snapshot View", deleted obsolete summary
# Version 0.7.3 JZ 20000126
#    print to file
# Version 0.7.2 IB 991112
#    corrected the determining of a relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    removing of HACKS, and correcting of relpath
# Version 2 RF 991108
#    rearangement of hash ordering and prelim dump
# Version 1 RF 991104
#    creation
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

use DBI;
require "$relpath/conf/site.pm";
require "$relpath/lib/filetools.pm";
require "$relpath/lib/localization.pm";

# must be the last require statement
require "$relpath/lib/custom-lib.pm";

## establish site-specific parameters
#
$site=setsite();

$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );

## establish connection in
#
$s = "$site->{'db_site_config'}"; 
$dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'}); 

$targetpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/stats";
$skeletonpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_locale_lang_def'}/admin/";
## Load templates for output HTML
#
#%template=();

########### general flow
#
# validation of input args ( can be order list )
#    fill in defaults for store/date
#
# select all sessions for date
#    determine sessions per referer
#    determine sessions per adbuy
#    determine sessions per artwork
#    determine sessions per affiliate
#    determine sessions per browser
# 
# select all pageviews
#    determine pagedepth per session
#    determine total number of page views
#
# select all orders
#    determine total revenues
#    determine total line items
#    determine total items
#    determine total shipping
#    determine total tax
#
# Derivative Values
#    average pagedepth = pageviews/sessions
#    conversion rate = sessions/orders
#    

## initialize tallying hash
#
%tot=();
$TOPTEN = 10;

## construct timestamps
#
if ( $#ARGV >= 2 ) {
   # there was a second argument, see if it is numeric...
   # TBD - what formats are acceptable?
   #	$test_str = $ARGV[1];
#	$year  = $test_str =~ m/.{4}$/  ;
	
#	if (($year < 1970) or ($year > 2100)) {
#			print "ERROR: invalid year format yyyymmdd\n";
#			exit 0;
#		}
### for now the number is the int days back
	$days = $ARGV[2];
}

#figure out the user who generated this file, default is cron
if ($ARGV[0]) {
	$statuser = $ARGV[0];
} else {
	$statuser = 'Cron';
}

# by default, this will run by cron and it does not have any argument
if ($ARGV[1]) {
	$time_n=$ARGV[1];
} else {
	$time_n=time();
}

# we do the stats every 24 hours in default 3:00am in the morning
## construct timestamp of midnight last
#
for (0..$days){

	if ($time_i) {$time_n = $time_i - (24 * 60 * 60);}
#	@ltime=localtime($time_n);
	$time_f=$time_n ; 
#	if ( ($ltime[0]+$ltime[1]+$ltime[2])!= 0 ) {
#		$time_i=$time_n - $ltime[0] - ( $ltime[1]*60 ) - ( $ltime[2]*3600 )  ;
#	} # done testing for midnight
## construct timestamp of midnight next
	$time_i=$time_f - ( 24 * 60 * 60);       # midnights are always 24hours apart

## sessions
#
# initialize session hash
	%session=();
# initialize counter hashes
	%ses_ref=();               # by referer
	%ses_adb=();               # by adbuy
	%ses_adb_pur=();           # made purchase
	%ses_art=();               # by banner artwork
	%ses_art_pur=();           # made purchase
#	%ses_aff=();               # by affiliate code
	%ses_bro=();               # by browser
	%ses_newu=();		       # by sid w/o UserID
	%ses_oldu=();		       # by sid w UserID
	$ses_newu{'TOTAL'} =0;
	$ses_oldu{'TOTAL'} =0;

	$q="SELECT \* from sessions where cdate>=$time_i and cdate<$time_f ";
	$sth = $dbh->prepare( $q );
	$sth->execute;
	$num=$sth->rows;
	$tot{'sessions_raw'}=$num;       # total sids burned through
	$tot{'sessions_ok'}=0;           # ignoring sessions from within site
	foreach ( 1..$num ) {
		$h = $sth->fetchrow_hashref;
		
		## filter out internally referred sessions
		#
		#next if ( $h->{'referer'} =~ /($site->{'siteurl'}|$site->{'sitesslurl'})/ );
		$tot{'sessions_ok'}++;
		
		## store session minimum data
		#
		$session{$h->{'sid'}}=$h;         # HACK - this will explode for large sessions...
		
		## test for existence of counters
		#
		if ( $h->{'sid'}) {
			if ( $h->{'browser'}) {
				if ( ! $ses_oldu{$h->{'sid'}} ){
					$ses_oldu{$h->{'sid'}} = 1;
					$ses_oldu{'TOTAL'}++;
				}
			}else{
				
				if ( ! $ses_newu{$h->{'sid'}} ){
					$ses_newu{$h->{'sid'}} = 1;
					$ses_newu{'TOTAL'}++;
				}	
			}
		}

		$ses_ref{$h->{'referer'}}++ if ( $h->{'referer'} );     # by referer
		$ses_adb{$h->{'adbuy'}}++ if ( $h->{'adbuy'} );         # by adbuy code
		$ses_art{$h->{'artwork'}}++ if ( $h->{'artwork'} ) ;    # by banner ad artwork
		$ses_bro{$h->{'browser'}}++ if ( $h->{'browser'} ) ;    # by customer browser

                ## TBD - add affiliate counting
	} # done with sessions

## start in on pageviews
#
	%tot_ref=();
	%tot_req=();

	$q="SELECT \* from clicks where cdate>=$time_i and cdate<$time_f ";
	$sth = $dbh->prepare( $q );
	$sth->execute;
	$num=$sth->rows;
	$tot{'pageviews_raw'}=$num;       # total pageviews burned through
	$tot{'pageviews_sid'}=0;          # pageviews with session id known
        $tot{'homepage'}=0;
	$tot{'refer'}=0;
	$tot{'request'}=0;

	foreach ( 1..$num ) {
		$h = $sth->fetchrow_hashref;
		$tot{'homepage'}++ if ( $h->{'request'} =~ /html\/home.shtml/ || $h->{'request'} eq "\/");
		@tmp = split(/\?/, $h->{'request'});
		$tot_req{$tmp[0]} ++;
		$tot{'request'}++;

		if ( $h->{'referer'} !~ /($site->{'siteurl'}|$site->{'sitesslurl'})/ ) {
			@tmp = split(/\?/, $h->{'referer'});
			$tot_ref{$tmp[0]} ++ if $h->{'referer'};
			$tot{'refer'}++;
		}

		next if ( ! $h->{'sid'} ); 

		$tot{'pageviews_sid'}++;   
		$session{$h->{'sid'}}->{'clicks'}=0 if (!$session{$h->{'sid'}}->{'clicks'});
		
		$session{$h->{'sid'}}->{'clicks'}++;
	} # done with pageviews
		
	
## start in on order lines (search ocart)
# login orders are placed in orders
# checked out orders are placed in ocart
# orders that are not checked out are place in tcart

	%tot_sku=();
	%sku_price=();

	$q="SELECT \* from ocart where cdate>=$time_i and cdate<$time_f ";
	$sth = $dbh->prepare( $q );
	$sth->execute;
	$num=$sth->rows;
	$tot{'orders_raw'}=$num;       # total pageviews burned through
	$tot{'orders_sid'}=0;          # pageviews with session id known
	foreach ( 1..$num ) {
		$h = $sth->fetchrow_hashref;
		$checkedout{$h->{'orderid'}} =1;

		# next if (! $orders{$h->{'sid'}});  #order not checked out
		$tot_sku{$h->{'msku'}} = 0 if ( ! $tot_sku{$h->{'msku'}});
		$tot_sku{$h->{'msku'}} += $h->{'QUANT'} ;
		$sku_price{$h->{'msku'}} = $h->{'UNITPR'} if ( ! $sku_price{$h->{'msku'}});

		next if ( ! $h->{'sid'} );
		#$orders{$h->{'sid'}}=1;
		if ($session{$h->{'sid'}}->{'adbuy'}) {
			if (! $ses_adb_pur{ $session{$h->{'sid'}}->{'adbuy'} }) {
		 		$ses_adb_pur{ $session{ $h->{'sid'} }->{'adbuy'} } =0;
			} 
			$ses_adb_pur{$session{$h->{'sid'}}->{'adbuy'}}++; 
			if (!$tot_adb{$session{$h->{'sid'}}->{'adbuy'}}) {
				$tot_adb{$session{$h->{'sid'}}->{'adbuy'}} = 0 ;
			}
			$tot_adb{$session{$h->{'sid'}}->{'adbuy'}} += $h->{'unitpr'}
		}	
		if ($session{$h->{'sid'}}->{'artwork'}) {
			if (! $ses_art_pur{ $session{$h->{'sid'}}->{'artwork'} }) {
		 		$ses_art_pur{ $session{ $h->{'sid'} }->{'artwork'} } =0;
			} 
			$ses_art_pur{$session{$h->{'sid'}}->{'artwork'}}++; 
		}	
	} # done with order line items

#start in on orders
#
        %tot_adb=();
        $q="SELECT \* from orders where cdate>=$time_i and cdate<$time_f ";
        $sth = $dbh->prepare( $q );
        $sth->execute;
        $num=$sth->rows;
        $tot{'orders_num'}=$num;       # total checkout entries (NOT finished orders)
        $tot{'orders_act'}=0;          # actual count of real orders
        $tot{'orders_tax'}=0;          # total sales tax collected
        $tot{'orders_shp'}=0;          # total shipping paid
        $tot{'orders_mds'}=0;          # total merchandise sold
        $tot{'orders_tot'}=0;          # total charged for orders
        foreach ( 1..$num ) {
                $h = $sth->fetchrow_hashref;
 		next if (! $checkedout{$h->{'orderid'}}); #checked out orders

                $orders{$h->{'sid'}} = 1; #record the order that was indeed checked out
 
                ## TBD - exclude uncompleted orders
 
                ## count revenue for the session
                #
                $session{$h->{'sid'}}->{'revenue'}=$h->{'ordtotal'};
 
                if ($session{$h->{'sid'}}->{'adbuy'}) {
                        $tot_adb{$session{$h->{'sid'}}->{'adbuy'}} += $h->{'ordtotal'};
                }
                if ($session{$h->{'sid'}}->{'artwork'}) {
                        $tot_art{$session{$h->{'sid'}}->{'artwork'}} += $h->{'ordtotal'};
                }
                $tot{'orders_tax'}+= $h->{'st_tax'};
#               $tot{'orders_shp'}+= $h->{'shipcost'};
                $tot{'orders_mds'}+= $h->{'st_taxable'} + $h->{'st_nontax'};
                $tot{'orders_tot'}+= $h->{'ordtotal'};
        } # done with orders     

#search for titles
	%title=();
	$q="SELECT msku, title from things";
	$sth = $dbh->prepare( $q );
	$sth->execute;
        $num=$sth->rows;
	foreach (1..$num) {
		$h = $sth->fetchrow_hashref;
		$title{$h->{'msku'}} = $h->{'title'};
	}

#number of people left stuff in cart and left site, search tcart
	%intcart=();
	$q = "SELECT sid from tcart where cdate>=$time_i and cdate<$time_f";
	$sth = $dbh->prepare( $q );
        $sth->execute;
        $num=$sth->rows;
	foreach (1..$num) {
                $h = $sth->fetchrow_hashref;
                $intcart{$h->{'sid'}} =1;
        } 

	foreach (keys %intcart) {$tot{'order_left'}++;}

## Output something...
#
	$fromtime = scalar localtime($time_i);
	$totime = scalar localtime($time_f);
	@fd = localtime($time_f);
	$fd[4] += 1;
	if ($fd[4] <10) { $filemonth = "0".$fd[4]; } 
	else { $filemonth = $fd[4] ;}

	$fd[3] = "0".$fd[3] if $fd[3] <10;
	$filedate = (1900 + $fd[5]).$filemonth.$fd[3];

	# check if the file for the day already exist
	opendir (DIR, $targetpath);
	@dirfiles = readdir (DIR);
	close (DIR);

	#check if cron and include directory exist
	foreach $d (@dirfiles) {
		if ($d eq 'cron') {
			$cronexist = 1;
		} elsif ($d eq 'include') {
			$includeexist = 1;
		}
	}

	if (! $cronexist) { system("mkdir $targetpath/cron"); }
	if (! $includeexist) { system("mkdir $targetpath/include"); }

	$snapflag = 1;
	if ($statuser eq $site->{'siteshortname'}) { #cron time
		$snapflag = 0;
		$statuser = 'crons';

		$targetpath .= "/cron";
	} else {
		$targetpath .= "/include";
	}

	#snapshot
	$snap = "$fromtime"."&"."$totime"."&"."$statuser";
	$totvisitors = $ses_newu{'TOTAL'} + $ses_oldu{'TOTAL'};

	$NVP{'FROMTIME'} = $fromtime;
	$NVP{'TOTIME'} =  $totime;
	$NVP{'STATSUSER'} = $statuser;

        $NVP{'SNAP_PAGE'} = $tot{'pageviews_raw'};
	$NVP{'SNAP_HOMEPAGE'} = $tot{'homepage'};
	$NVP{'SNAP_VISITOR'} = $totvisitors; 

	$snap .= "&"."$tot{'pageviews_raw'}"; 
	$snap .= "&"."$totvisitors";

	$NVP{'SNAP_LOG'} = $ses_oldu{'TOTAL'};
	$NVP{'SNAP_NOLOG'} = $ses_newu{'TOTAL'};
	$NVP{'SNAP_LEFT'} = $tot{'order_left'};

	if ($totvisitors) {
		$con_rate = $tot{'pageviews_raw'}/$totvisitors;
		@con_rate = split(/\./, $con_rate);
		$con_rate = $con_rate[0]."\.".substr($con_rate[1], 0, 2);
	} else {
		$con_rate = 'N/A';
	}
	$NVP{'SNAP_PER_VISITOR'} = $con_rate;
	$con_rate = $tot{'orders_num'} /$totvisitors if $totvisitors;
	$con_rate = substr($con_rate, 0, 6);
	$NVP{'SNAP_CONVERT'} = $con_rate;

#report
        $targ = "row_stats_report";
	$t = loadtemplate($targ, $om_render_lang, "html", 0);
	#top 10 request pages, title and URL
	$i = 0;
	foreach $pageurl (sort topreqsort keys %tot_req) {
		$i++;
		last if $i>=$TOPTEN;
		$viewpercent = $tot_req{$pageurl}/$tot{'request'} *100;
		@viewpercent = split(/\./, $viewpercent);
		$viewpercent = $viewpercent[0]."\.".substr($viewpercent[1], 0, 2);
		
		@page = split(/\//, $pageurl);
		@sku = split(/\./, $page[$#page]);
		if ($title{$sku[0]}) {
			$pagetitle = ucfirst($title{$sku[0]});
		} else {
			if ($sku[0]) { $pagetitle = ucfirst($sku[0]); }
			else { $pagetitle = 'Homepage'; }
		}

	    $pagerefurl = "http://$site->{'siteserver'}$site->{'sitedomain'}$pageurl";

	    $REP{'NUM'} = $i;
            $REP{'TITLE'} = "$pagetitle<br>";
	    $REP{'URL'} = $pagerefurl;
	    $REP{'NUMBER'} = $tot_req{$pageurl};
	    $REP{'PERCENT'} = $viewpercent;

	    $NVP{'TOPTENPAGE'} .= permutebit($t, \%REP);
	}
	
        #10 pages, title and URL
	$i = 0;
	foreach $pageurl (sort leastreqsort keys %tot_req) {
		$i++;
		last if $i>$TOPTEN;
		$viewpercent = $tot_req{$pageurl}/$tot{'request'} *100;
		@viewpercent = split(/\./, $viewpercent);
		$viewpercent = $viewpercent[0]."\.".substr($viewpercent[1], 0, 5);

		@page = split(/\//, $pageurl);
		@sku = split(/\./, $page[$#page]);
		if ($title{$sku[0]}) {
			$pagetitle = ucfirst($title{$sku[0]});
		} else {
			if ($sku[0]) { $pagetitle = ucfirst($sku[0]); }
			else { $pagetitle = 'Homepage'; }
		}
		$pagerefurl = "http://$site->{'siteserver'}$site->{'sitedomain'}$pageurl";

		$REP{'NUM'} = $i;
                $REP{'TITLE'} = "$pagetitle<br>";
                $REP{'URL'} = $pagerefurl;
                $REP{'NUMBER'} = $tot_req{$pageurl};
                $REP{'PERCENT'} = $viewpercent;   

                $NVP{'LEASTTENPAGE'} .= permutebit($t, \%REP); 
	}

        #10 browsers
	$i = 0;
	foreach $pageurl (sort topbrosort keys %ses_bro) {
		$i++;
		last if $i>$TOPTEN;
		$viewpercent = $ses_bro{$pageurl}/$totvisitors *100;
		@viewpercent = split(/\./, $viewpercent);
		$viewpercent = $viewpercent[0]."\.".substr($viewpercent[1], 0, 2);

		$NVP{'TOPTENBROSWER'} .= "<tr><td>$i\. $pageurl</td> <td align=center> $ses_bro{$pageurl} </td> <td align=center>$viewpercent\%</td></tr>\n";
	}

    #top 10 Refer pages, title and URL
	$i = 0;
	foreach $pageurl (sort toprefsort keys %tot_ref) {
		$i++;
		last if $i>$TOPTEN;
		$viewpercent = $tot_ref{$pageurl}/$tot{'refer'} *100;
		@viewpercent = split(/\./, $viewpercent);
		$viewpercent = $viewpercent[0]."\.".substr($viewpercent[1], 0, 2);

                $REP{'NUM'} = $i;
                $REP{'TITLE'} = "";
                $REP{'URL'} = $pageurl; 
                $REP{'NUMBER'} = $tot_ref{$pageurl};
                $REP{'PERCENT'} = $viewpercent; 

		$NVP{'TOPTENREFER'} .= permutebit($t, \%REP);
	}

#order information
	$NVP{'ORDER_NUM'} = $tot{'orders_num'};
	$NVP{'ORDER_TOTAL'} = $tot{'orders_tot'}; 
	$NVP{'ORDER_SHIP'} = $tot{'orders_shp'};
	$NVP{'ORDER_TAX'} = $tot{'orders_tax'};

	$snap .= "&"."$tot{'orders_num'}";
	$snap .= "&"."$tot{'orders_tot'}";
	
	#5 items
        $targ="row_stats_topitem";
        $t = loadtemplate($targ, $skeletonpath);
	$i = 0;
	$snap .= "&"; 
	foreach $sku (sort topskusort keys %tot_sku) {
		$i++;
		last if $i>5;
		my $sku_sale = $sku_price{$sku}* $tot_sku{$sku};
		$skuurl = "http://$site->{'siteserver'}$site->{'sitedomain'}/html/$sku\.shtml";
		$ITEM{'NUMBER'} = $i;
		$ITEM{'SKUURL'} = $skuurl;
		$ITEM{'SKU'} = $sku;
		$ITEM{'TOT_SKU'} = $tot_sku{$sku};
		$ITEM{'PRICE'} = $sku_price{$sku};
		$ITEM{'SALE'} = $sku_sale;

		$NVP{'TOPITEMS'} .= permutebit ($t, \%ITEM);
		$snap .= permutebit ($t, \%ITEM); 
	}

#Advertising Summary

	$targ="row_stats_ad";
	$t = loadtemplate($targ, $skeletonpath); 

   #adbuy
	foreach $adbuy (sort adbuysort keys %ses_adb) {
		$con_rate = $ses_adb_pur{$adbuy} / $ses_adb{$adbuy} if $ses_adb{$adbuy};
		$con_rate = substr($con_rate, 0, 6);
		my $avesale = $tot_adb{$adbuy} / $ses_adb_pur{$adbuy} if ($ses_adb_pur{$adbuy});
		$AD{'AD'} = $adbuy ;
		$AD{'VISITOR'} = $ses_adb{$adbuy};
		$AD{'ORDER'} = $ses_adb_pur{$adbuy};
		$AD{'TOTAL'} = $tot_adb{$adbuy};
		$AD{'AVESALE'} = $avesale;
		$AD{'CONVERSION'} = $con_rate;
		
		$NVP{'ADBUY_ROW'} .= permutebit($t, \%AD);
	}

   #artwork
	foreach $artbuy (sort artsort keys %ses_art) {
		$con_rate = $ses_art_pur{$artbuy} / $ses_art{$artbuy} if $ses_art{$artbuy};
		$con_rate = substr($con_rate, 0, 6);
		my $avesale = $tot_art{$artbuy} / $ses_art_pur{$artbuy} if ($ses_art_pur{$artbuy});

		$AD{'AD'} = $artbuy;
                $AD{'VISITOR'} = $ses_art{$artbuy};
                $AD{'ORDER'} = $ses_art_pur{$artbuy};
                $AD{'TOTAL'} = $tot_art{$artbuy}; 
                $AD{'AVESALE'} = $avesale;
                $AD{'CONVERSION'} = $con_rate;
 
                $NVP{'ARTWORK_ROW'} .= permutebit($t, \%AD);
	}
}

#write to the stats file
$targ = "os_allcommerce-runstats";
$l = loadtemplate($targ, $skeletonpath);
$l = permutebit($l, \%NVP);
open(STAT, ">$targetpath/$filedate.html");
print STAT $l;
close(STAT);

$snap .= "&"."$filedate";
print STDOUT $snap if ($snapflag);

$sth->finish if ( $sth );
$dbh->disconnect if ( $dbh );

#sort rules
sub topreqsort  { $tot_req{$b} <=> $tot_req{$a}; }
sub leastreqsort { $tot_req{$a} <=> $tot_req{$b}; }
sub toprefsort { $tot_ref{$b} <=> $tot_ref{$a}; }
sub topbrosort { $ses_bro{$b} <=> $ses_bro{$a}; }
sub adbuysort { $ses_adb{$b} <=>  $ses_adb{$a}; }
sub artsort { $ses_art{$b} <=>  $ses_art{$a}; }
sub topskusort { $tot_sku{$b} <=>  $tot_sku{$a}; }
1;
