#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#  validate.pm
#  RF 981123
#  sku and field level validation routines
# Version 0.7.27 TG 20000606
#    added check for anything but integers in validatefield for manweight and added
#    check for linkwt_use
# Version 0.7.26 TG 20000531
#    replaced validatesku to ValidateOBJID
#    needs to be fixed to support multi language
#    added optional parameter to ValidateOBJID to check for product type and forced it in validatelink
# Version 0.7.25 GF 20000530
#    validatefield corrected
#    added validatbrand
#    changed validatesku for secondary sku use
# Version 0.7.24 JN 20000524
#    added firstname lastname to validateaddr
#    also changed the phone error msg
# Version 0.7.23 GF 20000523
#    validatesku
# Version 0.7.22 JN 20000522
#    add validate_payment
#    added ccvs support
# Version 0.7.21 GF 20000502
#    validate email error messages
# Version 0.7.19 IB 20000404
#    changed store to site
# Version 0.7.18 IB 20000302
#    removed -w
# Version 0.7.17 IB 20000301
#    updated copyright
# Version 0.7.16 JZ 20000214
#    email address validation
# Version 0.7.15 GF 2000 
#    changed sku length hard codes
#    cleaned up error messages
# Version 0.7.14 JZ 20000114
#    date validation fixed
# Version 0.7.13 JN 20000111 
#    sku_prelen checks against store.pm
#    UID length check now to field length
# Version 0.7.12 JZ 20000110
#    validate of date to be Y2K complient
# Version 0.7.11 IB 20000108
#    fixed the SKU validation messages
# Version 0.7.10 JN 991222
#    changed the checking of a three-letter code
# Version 0.7.9 JN 991217
#    tightened up the AXISNAMES error detection
# Version 0.7.8 IB 991217
#    removed the ERLOG
# Version 0.7.7 IB 991128
#    corrected setstore
# Version 0.7.6 IB 991113
#    preparation for release
# Version 0.7.5 IB 991113
#    corrections to global parameters  
# Version 0.7.4 IB 991110
#    corrected database calls
# Version 0.7.1 SH 990830
#    got rid of zip trapping
# Version 11 SH 990708
#    Done w/ some system 6 retrofitting, run_payment & validatefield
#    Need to clean up validatefield to get rid of irrelevant fields  
# Version 10 - 981215
#    run_payment operational
# Version 9 - 981214
#    updated validate_payment with much better sniffing
# Version 8 - 981212
#    replaced validate_payment( \% ):\%    
#    added run_payment stub
# Version 7 - 981211
#    added validate_payment( \% ):\% - validates payment fields in form
# Version 6 - 981210
#    added validateaddr( \% ):$err - validates address
#    added address fields to ValidateField
# Version 5 - 981207 
# Version 4 - 981206
#    added validateemail
#    advanced from version 2 of 981124, version 3 has db trouble
# Version 3 - 981205
#    derelicted RF 981206
# Version 2 - 981124
#    added validatelink
#

 
## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in cgi-bin/om,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;
 
@rpath=@temp;
 
$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
 
$rundir= join "/", @rpath;
chdir( $rundir );
 
$relpath="";
$siteshortname="";
## short circuit if $s does not contain cgi-bin
if ( $s !~ /cgi-bin/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case
 
while ( @temp ) {
   $u = pop @temp;
 
   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels
 
   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );
 
   if ( $u eq "cgi-bin" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - cgi-bin is off the site root, so the next up is site root
 
} # done constructing relative path to store root
 
require "$relpath/conf/site.pm";

##
# Uncomment the ccvs.pm require to use the expiremental ccvs
# payment processing library
#
#require "$relpath/lib/ccvs.pm";
$site = setsite();          

##### run_payment( \% ) : \% - takes order,card data,
#                              returns trid and response
# 20000525 JN
#  Removed Stub message. Now the command for run payment
#  is based on the return of getPaymentProcess

sub run_payment {
   my( $z,$o,$dbs,$sts,$q,$time,$trid );
   $z = $_[0];             # incoming parameters
   $o={};                  # initialize return hashref
   $paymenttype = getPaymentProcess($z->{'CCNAME'});
   $o = &{"run_$paymenttype"}($z);
   return $o;
} #/run_payment

##### getPaymentProccess 
# 
# JN 20000522
# this function return the name of the payment process
# function to use
# 
# Requires:
# 	CCNAME to equal the name of the credit card 

# Note: right now it is hard coded to return the site variable
# 	payprocess
sub getPaymentProcess {

   my ($ccname,$r);
   $ccname = $_[0];
   
   $r = $site->{'payprocess'};

   return $r;

}

#### run_none
#
# JN 20000606
#
# this is a function for no payment processing
# it will always return a success


sub run_none {

  my ($ret_hash) = $_[0];
  $ret_hash->{'STATUSDVALUE'} = 'success';
  return $ret_hash;

}

##### ValidatePayment( \% ):\% - takes form stuff, validates
#         (minimallistically) the information
#     Rev 2 981214
sub validate_payment {
   my( $h,$o,@temp,$k,$v,$err,$f,@dig,$st,$d,$x,$y );
   $h=$_[0];
   $err=""; 
   $f='CCID_inp,CCEXPMO_inp,CCEXPYR_inp,CCNAME_inp,CCPHONE_inp';

   $o={};                 # put new keys here...
   @temp = keys %{$h};
   while ( @temp ) {
      $k=shift @temp;
      next if ( $f !~ /$k/ );
      $o->{$k}=$h->{$k};
      if ( $k eq "CCNAME_inp" ) {            # perform checksum if name...
         $o->{$k} =~ tr/a-z/A-Z/;   # cannonicalize to uppercase         
         $o->{$k} =~ tr/ //d;       # remove spaces
	 if (!$o->{$k}) { $err="Please pick a credit card name. ";
			  $o->{'CCNAME_err'}="Please pick a credit card name. "; }
      } elsif ( $k eq "CCID_inp" ) {         # minimalist error checks
         $o->{$k} =~ tr/0-9//cd;    # strip all non-numerics
         ## construct checksum for all cards
         @dig = split /\s*/, "$o->{$k}";
         $st=0;
         $d=1;
         while (@dig) {
            $x=pop @dig;
            $y= $x*$d ;
            if ( $y > 9 ) {   # need sum of digits, first digit is 1 ...
               $y = ( $y - 10 ) + 1;
            } # done checking for 2-digit $y
            $st+= $y;
            $d++;
            $d=1 if ( $d>2 );
         } # done running through digits

         ## see if plausible checksum
         $y = $st - int($st/10)*10;
         if ( $y!=0 ) { # the card number is not plausible
            $o->{'CCID_err'} = "Invalid credit card number.  Please enter.";
            $err="There were problems with your payment";
         } else {       # check for reasonable number of digits
            if ( length($o->{$k})==16 ) {       ## MC,Visa,Discover
               if ( $h->{'CCNAME_inp'} eq "MC" ) {
                  if ( substr($o->{$k},0,1) ne "5" ) {
                     $o->{'CCID_err'}="Invalid MasterCard Number.";
                     $err="There were problems with your payment";
                  } # else plausible...
               } elsif ( $h->{'CCNAME_inp'} eq "VISA" ) {
                  if ( substr($o->{$k},0,1) ne "4" ) {
                     $o->{'CCID_err'}="Invalid Visa Number.";
                     $err="There were problems with your payment";
                  } # else plausible...
               } # what else has 16 digits?
            } elsif ( length($o->{$k})==15 ) {
               # Amex, no data..
            } elsif ( length($o->{$k})==14 ) {
               # Visa?
               if ( substr($o->{$k},0,1) ne "4" ) {
                  $o->{'CCID_err'}="Invalid Visa Number!";
                  $err="There were problems with your payment";
               } # else plausible...
            } elsif ( length($o->{$k})==13 ) {
               # old cards, no data...
               # Visa?
               if ( substr($o->{$k},0,1) ne "4" ) {
                  $o->{'CCID_err'}="Invalid Visa Number!";
               $err="There were problems with your payment";
               } # else plausible...
            } else {                            ## no known cards 
               $o->{'CCID_err'}="Credit card number not valid.";
               $err="There were problems with your information";
            } # done with length permutations...            
         } # done checking cc number for plausibility 
      } elsif ( $k eq "CCPHONE_inp" ) {
         $o->{$k} = $h->{$k};
         $o->{$k} =~ tr/0-9//cd; 
         if (( length($o->{$k})==10 ) 
         or  ( length($o->{$k})==11 ) ) {
            # this is OK unless a 900 number...
            if ( substr($o->{$k},0,4) =~ /900/ ) {
               # this should NOT be a 900 number
               $o->{'CCPHONE_err'} = 
               "Customer Service 900 numbers are still very rare";
               $err="There were problems with your information";
            } # end of ruling out 900 numbers
         } else {
            $o->{'CCPHONE_err'} = 
            "This information is required as a security precaution";
            $err="There were problems with your information";
         } # done examining the phone number...   
      } elsif ( $k eq "CCEXPMO_inp" ) {
         if (( $h->{$k} > 12 ) or ( $h->{$k} < 1 )) {
            $o->{'CCEXPMO_err'} = 
            "Please select month the credit card expires.";
            $err="There were problems with your information";
         }#
      } elsif ( $k eq "CCEXPYR_inp" ) {
         if ( (( $h->{$k} < 97 ) and ( $h->{$k} > 06 )) 
         or ( length($h->{$k}) < 2 ) ) {
            $o->{'CCEXPYR_err'} = 
            "Please select the year the credit card expires.";
            $err="There were problems with your information";
         }#
      } # no "else", irrelevant field... 
   } # done validating the payment fields

   ## check month and year OK
   #
   ($mo,$yr) = ( localtime(time) )[4..5];
   $yr+=1900; 
   $mo++;
   if ( $h->{'CCEXPYR_inp'} > 95 ) {
      $y2 = 1900 + $h->{'CCEXPYR_inp'};
   } elsif ( length($h->{'CCEXPYR_inp'}) > 0 ) {
      $y2 = 2000 + $h->{'CCEXPYR_inp'};
   } else { 
      $y2=0;
   } # done with variations on date...
   if ( $yr == $y2 ) {
     if ( $mo > $h->{'CCEXPMO_inp'} ) {
         $err="There were problems with your information";
         $o->{'CCEXPYR_err'} = "This credit card has expired.";
     } # else no error
   } # done checking for month, year in the past

   ## error state and messages
   #
   if ( length($err)>2 ) {
      $o->{'CC_err'} = $err;
      $o->{'CC_ok'} = 0;
   } else {
      $o->{'CC_ok'} = 1;
   } # done with the error state and messages


   return $o;
} #/validate_payment
#========================================================================

##### ValidateField
##        takes fieldname, value, hash of sku
sub validatefield {
   my( $f,$v,$h,$rv,$testmode,$q,$sth );
   ( $f,$v,$h ) = @_;
 
   $testmode=1;
 
   if ($testmode) {
      open FX, ">>/tmp/validatefield";
      print FX "field $f value $v";
   }
 
   $rv="";
 
#   if ( $f eq "sku" ) {
#      $rv=validatesku($v);
#     # done with SKU
#   } 


  if ($f eq "obj2") {
     # only validate if one was entered in.  Ignore blank entries for this
  if ($v) {
     $rv=ValidateOBJID($v, $dbh, 1);         # Tara changed to tell it to check for valid object IDs!
     # MULT_LANG - This needs to be changed to support multiple languages
     if ($rv eq "INVALID_OBJ") {
        $rv = "The object entered could not be found!\n";
     }
 }
 } elsif ( $f eq "alttitle" ) {
      # should be longer than 4 characters and less than 40
      #    or zero characters
      #
      if ( (( length($v)>3 ) and (length($v)<41))
      or ( length($v)==0 )) {
         # get a warm fuzzy feeling, it is OK
      } else {
         # grouse at them
         $rv=$v;
         $v=length($v);
         $rv.="Error - Alternate title should be 0 or 4 to 40 chars";
      }
      # done with ALTTITLE
 
   } elsif ( $f eq "sku_ref" ) {
 
      # this needs to be a unique letter code if the
      # sku is a brand or series header           
      #
 
#      if ((( $h->{'SKUKIND_inp'} =~ /[BbXx]/ )
#      or ($h->{'BRAND_inp'} =~ /xbr.0000/ ))
#      and ( $h->{'newflag'} =~ /Y/i)) {
         if (length($v) == $site->{'site_sku_prelen'}) {
 
            # this is a brand or series header
            $q="SELECT objid,fieldname,body from extraprop where ";
            $q.=" fieldname='sku_ref' and body LIKE '$v%' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
 
            if ( $sth->rows > 0 ) {
               $rv="Error - $site->{'site_sku_prelen'} letter code not unique! ";
            }
 
            $sth->finish;
 
         } else {
            $rv="Error - Must be $site->{'site_sku_prelen'} letters";
         }
 
   } elsif ( $f eq "axisnames" ) {
      if (length($v)!=$site->{'site_sku_totlen'}) {
      @newv = split /:/, $v;
      foreach $x (@newv) {
             if (length($x)!=$site->{'site_sku_totlen'}) {
                $rv = "Error - Values must be $site->{'site_sku_totlen'} characters long";
             } else {
            $rv=validatesku($x) if (!$rv);
             }
          }
          if ($#newv > 3) {
              $rv = 'Error - Maximum of 4 skus alowed';
          }
 
      } else {
         $rv=validatesku($v);
      } # done with AXISNAMES
 
   } elsif ( $f eq "body" ) {
      # no restrictions?
                                                   
   } elsif ( $f eq "TERM" ) {
       if (length($v) < 2) {
       $rv="Error - Term must be at least two characters long";
       }
   } elsif ( $f eq "SEE_TERM" ) {
       if (length($v) < 2) {
       $rv="Error - Term must be at least two characters long";
       }
 
   } elsif ( $f eq "USE_TERM" ) {
       if (length($v) < 2) {
       $rv="Error - Term must be at least two characters long";
       }
 
   } elsif ( $f eq "brand" ) {
      # should be a valid objid
      if ( $rv=validatebrand($v) ) {
      $rv="Error - This does not appear to be a good brand.";
      }
 
      # done with BRAND
 
   } elsif ( $f eq "brief" ) {
      # should be longer than 5 characters and less than 250
      if (( length($v)<5 ) or (length($v)>250)) {
            if ( length($v)>250 ) {
               $rv="Error - Brief should be less than 250 characters";
            } else {
            $rv="Error - Brief should be between 5 and 250 characters";
            }
      }# done with BRIEF
 
#glenn
 
   } elsif ( $f eq "fieldname" ) {
      # should be element of enumerated list...
 
      if ( $v !~ /^(DESCRIPT|OVERFLOW)/ ) {
         if ( length($v)>0 ) {
            $rv = "Error - Content_type not on list...";
         } else {
            $rv = "Error - Content_type blank";
         }
      }
      # done with CONTENT_TYPE                    
   } elsif ( $f eq "COST" ) {
      # should be positive
      if ( $v<0 ) {
         $rv="Error - Unit Wholesale Cost ";
         $rv.=" should be zero or positive";
      }
      # done with COST
 
   } elsif ( $f eq "manweight" ) {
      # should be a number 01-99, or blank
       if ($v =~ /(\D)+/) {
	   $rv="Error - Weight should be blank or from 1 through 99";
       }
      if ($v<0) {
         $rv="Error - Man weight should be a positive number";
      } elsif ($v>99) {
         $rv="Error - Man weight should be blank or from 1 through 99";
      }
      # done with Manual Link Weighting
    }  
   elsif ( $f eq "linkwt_use" ) {
      # should be a number 01-99, or blank
     if ($v =~ /(\D)+/) {
	 $rv="Error - Weight should be blank or from 1 through 99";
     }
      if ($v<0) {
         $rv="Error - Weight should be a positive number";
      } elsif ($v>99) {
         $rv="Error - Weight should be blank or from 1 through 99";
      }
      # done with Manual Link Weighting
    }  
   elsif ( $f eq "MSRP" ) {
      # should be positive
      if ( $v<0 ) {
         $rv="Error - Manufacturer's Suggested Price";
         $rv.=" should be zero or positive";
      }
      # done with MSRP
 
   } elsif ( $f eq "RANKING" ) {
      # should be a number 01-99
      if ($v<1) {
         $rv="Error - RANKING should be a positive number";
      } elsif ($v>99) {
         $rv="Error - RANKING should be from 1 through 99";
      }
      # done with ranking on 1-99 scale
 
   } elsif ( $f eq "REGPRICE" ) {
      # should be positive
      if ( $v<0 ) {
         $rv="Error - Regular Selling Price ";
         $rv.=" should be zero or positive";
      }
 
   } elsif ( $f eq "repl_by" ) {
      if (length($v)==8) {
         $rv=validatesku($v);
      }
      # done with REPL_BY
 
  }  elsif ( $f eq "see_sku" ) {
      if (length($v)==$site->{'site_sku_totlen'}) {
         $rv=validatesku($v);
     } else {$rv="Error - Sku must be of length $site->{'site_sku_totlen'}";} 
      # done with SEE_SKU
 
  } elsif ( $f eq "use_sku" ) {
      if (length($v)==$site->{'site_sku_totlen'}) {
      $rv=validatesku($v);
      } else {$rv="Error - Sku must be of length $site->{'site_sku_totlen'}";}
  } elsif ( $f eq "OBJ1" ) {
      $rv=validatesku($v);
      # done with SKU1
 
   } elsif ( $f eq "obj2" ) {
      $rv=validateobjid($v);
      # done with SKU2
 
   } elsif ( $f eq "title" ) {
      # should be longer than 3 characters and less than 80
      if (( length($v)<3 ) or (length($v)>80)) {
         $v=length($v);
         $rv.="Error - Title should be between 3 and 80 characters";
      }
 
      # done with TITLE
 
   } elsif ( $f eq "vcode" ) {
      # should be a two character string or blank
      if ($v && $v ne "none") {
      @newv= split /:/, $v;
      foreach $x (@newv) {
          if ( length($x) != 3) {
          $rv="Error - Variant code should be 3 characters long";
          }
      }
      }
      # end of variant code
 
  } elsif ( $f eq "PASSWD") {
      if ((length($v) > 20) || (length($v) < 3 )) {
      $rv="Error - Passwords should be between 3 to 20 characters";
      }
  } elsif ( $f eq "UID") {
      if ((length($v) > 40) || (length($v) < 3)) {
      $rv="Error - UIDs should be between 3 to 40 characters";
      }
  } elsif ( $f eq "UNAME") {
      if ((length($v) < 1)) {
      $rv="Error - you need to enter user name";
      }
  } elsif ( $f eq "SDATE") {                               
      $v =~ s/\s//g;
      if ($v !~ /^\d{1,2}\/\d{1,2}\/\d{4}$/) {
          $rv="Error - Date must be of form mm/dd/yyyy";
      }else {
                my @rrv = split /\//, $v;
                if ($rrv[0]>12 || $rrv[1]>31) {
                    $rv="Error - Date was entered invalid";
                  }
      }
  } elsif ( $f eq "SHIPPER" ) {
      if ((length($v) < 1)) {
          $rv="Error - you must enter a valid shipper name";
      }
  } elsif ( $f eq "TRACKING") {
      if ((length($v) < 1)) {
          $rv="Error - you must enter a tracking number";
      }
  }
   if ( $testmode ) {
      if ($rv) {
         print FX " error $rv\n";
      } else {
         print FX "\n";
      }
      close(FX);
   } # end of testmode
 
   return $rv;
 
} #/validatefield
 
 
##### ValidateSKU
#
# Version 2 20000525 GF
#
sub validatesku {
   my( $v,$sth,$q,$rv,$num );
 
   $v=$_[0];
   $rv="";
 
   # this needs to be an 8 digit alphanumeric
   # with the 4th digit being the store code
 
   $v =~ tr/A-Za-z0-9//cd;   # strip everything but alphanumerics
   $v =~ tr/A-Z/a-z/ ;       # lowercase it
   if ( length($v)==0 ) {                           
      $rv="Error - SKU is blank!";
   } elsif ( length($v)!=8 ) {
      $rv="Error - SKUs are 8 characters long!";
   } elsif ( $v !~ /[a-z]{3}($site->{'sitecode'})[0-9]{4}/ ) {
      $rv="Error - SKUs are 3 characters, a store code,";
      $rv.=" then four digits" ;
   } else {
      # check to see if it exists...
      $q = "SELECT SKU from things where SKU='$v'";
      $sth=$dbh->prepare($q);
      $sth->execute;
      $num=$sth->rows;
      if ( $num == 0 ) {
         $rv="Error - nothing found";
         $sth->finish;
      } elsif ( $num > 1 ) {
         $rv = "Error - your SKU should have one record, found ";
         $rv.= $sth->rows." records!";
         $sth->finish;
      } else {
         $sth->finish;
      }
   }
 
   return $rv;
} #/validatesku
 
 
## validatebrand - takes a valid brand and returns err
#
# Version 1 20000525 GF
#
sub validatebrand {
 my ($objid,$nums,$letters,$num,$sts,$err);
 
  $objid=$_[0];
  $err="";
 
  if (length ($objid) == 0) {
      $err = 1;
      return $err;
  }
 
  ($nums,$letters) = split (/-/,$objid);
 
  if ((length $nums == 0) || (length $letters == 0)) {
     $err = 1;
    return $err;                                
  }
 
  if ($nums =~ /(\D)+/) {
     $err = 1;
     return $err;
  }
 
  if ($letters =~ /([^a-zA-Z])+/) {
     $err = 1;
     return $err;
  }
 
  ## determine identity of next page in checkout sequence
  #
  $q="SELECT objid from things where objid='$objid' ";
  $sts = $dbh->prepare( $q );
  $sts->execute;
  $num=$sts->rows;
 
  if ( $num==1 ) {
     $err='';
     return $err;
 
  } else {
     $err = 1;
     return $err;
  }
 
} #/validate brand
 
sub ValidateOBJID {
  my $objid = $_[0];
  my $dbh = $_[1];
  my $check_product = $_[2];     # optional parameter - if set, then check if the product is 134-A or 136-A
 
  my ($nums, $letters, $err);
 
  if (length ($objid) == 0) {
      $err = "INVALID_OBJ";
      return $err;
  }
 
  ($nums,$letters) = split (/-/,$objid);
 
  if ((length $nums == 0) || (length $letters == 0)) {
      $err = "INVALID_OBJ";
    return $err;
  }                                        

  if ($nums =~ /(\D)+/) {
      $err = "INVALID_OBJ";
    return $err;
  }
 
  if ($letters =~ /([^a-zA-Z])+/) {
      $err = "INVALID_OBJ";
      return $err;
  }
 
  $sth = $dbh->prepare(q{
    select objid, ref_from, msku, vcode, objclass from things where objid = ?})
    || warn "Cannot prepare " . $dbh->err;
 
  $sth->execute($objid)
    || warn "Cannot execute " . $dbh->err;
 
  $sth->bind_columns(\$t_objid, \$ref_from, \$msku, \$vcode, \$objclass);
  $sth->fetchrow();
  if($t_objid eq "" ) {
    $err = "INVALID_OBJ";
    return $err;
  }
  
  # if we are supposed to check for valid product, check now
  if (($check_product) && (($objclass ne "134-A") && ($objclass ne "136-A"))) {
      $err = "INVALID_OBJ";
      return $err;
  }
}
      

##### ValidateLink
## validatelink( \% ):\% -- validate the data in a link
#
sub validatelink {
   my($h,$i,@temp,$k,$v);
   $h=$_[0];

   ## adds two fields as well as error messages
   #    changestowrite - "yes|no" if there are changes to write
   #    validationcheck - "ok|-" if the record passes validation 
   $h->{'validationcheck'}="ok";   # default
   $h->{'changestowrite'}="no";   # default

   ## run through keys and validate _inp 
   #
   foreach ( keys %{$h} ) {
      next if ( ! /_inp$/ );
      $k = $_;
      $k =~ s/_inp$//;

      $v = validatefield( $k, $h->{$_}, $h );
      if ( $v ) {
         ## it failed to pass validation
         #
         if (( $k eq "OBJ2" ) 
         and ( $h->{'ROWID'} =~ /^newrow/ )) {  # no error msg
            $v="" if ( $v =~ /blank/ );         # for blank skus on
         }                                      # new rows...
         $k.="_err";
         $h->{$k}=$v;
         $h->{'validationcheck'}="";
      } else {
         ## it passed, see if there is something to write
         #
         $k.="_ini";
         $h->{'changestowrite'}="yes" if ( $h->{$_} ne $h->{$k} );
      } # done examining data
   } # end of running through hash keys

   ## just in case...
   $h->{'changestowrite'}="no" if ( $h->{'validationcheck'} ne "ok" );

   ## cleanup
   #
   undef @temp;
   return $h;

} #/validatelink


##### validateemail( s ):s  -- returns blank or error
# RF 981206
sub validateemail {
   my($email,$rv);
   $email = $_[0];
   $rv="";
   if ( ! $email ) {
      # Error - no email address entered!
      $rv="a";
      return $rv;
   };

   if ($email !~ /\@/) {
      # Error - Wrong email address format, need \@
      $rv ="b";
      return $rv;
   };

   @e = split "\@", $email;

   if ($e[0] !~ /(\w+)/ || $e[1] !~ /\w+\.\w+/ || length($e[1])< 4 ) {
      # Error - Invalid Email address!
      $rv ="c";
      return $rv;
   }

   return $rv;
} #/validateemail


##### validateaddr( \% ): \% - returns null or errored address hashref
# RF 981210
sub validateaddr {
   my( $f,%i,%addr,$k,$k2,$err,$e );
   %i=();
   %i = %{$_[0]} if ( $_[0] );
   
   # fieldlist
   $f="addrid,status,firstname,lastname,address1,address2,city,county,state,postcode,"; 
   $f.="country,dayphone";

   $err="";
   # build address hashref
   foreach $k ( keys %i ) {
      $kr=$k;
      $kr=~s/(_inp|_pre|_ini|_err)$// ;
      next if ( $f !~ /$kr/ );
      $addr{$k}=$i{$k};
      next if ( $k !~ /_inp$/ );

      # run the validation
      $k2 = $kr."_err";
      $e="";
      ## quick decision tree 
      #     based on field name
      if ( $k =~ /^firstname/ ) {
         $e="Error -  Please enter your first name here, must be at least two letters" 
         if (length($i{$k})<2 );
      } elsif ( $k =~ /^address1/ ) {
         $e="Error - Please enter street number and name here"
         if (length($i{$k})<2);
     } elsif ( $k =~ /^lastname/ ) {
         $e="Error - Please enter your last name here, must be at least two letters"
         if (length($i{$k})<2);
     } elsif ( $k =~ /^city/ ) {
         $e="Error - Please enter the name of the city"
         if (length($i{$k})<2);
      } elsif ( $k =~ /^state/ ) {
         $e="Error - Please select your state from this list"
         if (length($i{$k})<2);
      } elsif ( $k =~ /^postcode/ ) {
         $e="Error - Please enter at least your 5 digit zipcode "
         if (length($i{$k})<5);

      } elsif ( $k =~ /^dayphone/ ) {
         $addr{$k}=~tr/-\(\)[]{}.A-Za-z //d;
         $e="Error - Please your telephone number including your Area Code" 
          
         if ((length($addr{$k})!=10)  
         and (length($addr{$k})!=11));
      } # done with fields we care about

      $addr{$k2}=$e if ( $e ); 
      $err.=$e if ( $e ); 
   } # done constructing address href
   $addr{'errors'}=$err if ( $err );

   return \%addr;
} #/validateaddr

1;
