#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#  variant.pm
#
# Version 1.0.0 TG created the first version
# Version 1.0.1 TG 
#    added getVariant
# Version 1.0.2 TG
#     added support for multi-variants
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in cgi-bin/om,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;
 
@rpath=@temp;
 
$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
 
$rundir= join "/", @rpath;
chdir( $rundir );
 
$relpath="";
$siteshortname="";
## short circuit if $s does not contain cgi-bin
if ( $s !~ /cgi-bin/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case
 
while ( @temp ) {
   $u = pop @temp;
 
   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels
 
   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );
 
   if ( $u eq "cgi-bin" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - cgi-bin is off the site root, so the next up is site root
 
} # done constructing relative path to store root
 
require "$relpath/conf/site.pm";

##
# Uncomment the ccvs.pm require to use the experimental ccvs
# payment processing library
#
#require "$relpath/lib/ccvs.pm";
$site = setsite();          

use DBI;
require "$relpath/lib/formtools.pm";

$debug = 1;
if ($debug) {
    open VAR_ERLOG, ">/tmp/variant";
}

# Routine used to get title and any extra fields requested. 
# Gets the title for variant data based on the master and the 
# body of the extraprop table
# Expects that only ONE row will be returned
sub get_fields_and_title {
    my ($fields) = $_[0];
    my ($where_clause) = $_[1];
    my ($objid) = $_[2];
    my ($dbh) = $_[3];

    my ($q, $sth, @results, @extraprop, @master_row, @rows, $row, $title);

    # Added axisnames, ref_from and vcode to populate title of variant
    $q="SELECT title, axisnames, ref_from, vcode";

    # if there are additional fields append them
    if ($fields) {
	$q .= ", $fields";
    }

    # Since objid is the primary key, only one row will be returned from this query
    $q .= " from things where objid='$objid'";

    # if there are additional where clause cases, append them
    if ($where_clause) {
	$q .= " $where_clause";
    }

    if ($debug) {
	print VAR_ERLOG "Select from things with --> $q\n";
    } 

    $results = &query($q, $dbh);
       
    if ( $results->{'numrows'} ) {
	if ($results->{'ref_from'} ne $objid && $results->{'ref_from'} ne "") {
	    if ($debug) {
		print VAR_ERLOG "Need to set objid variant title!\n";
	    }

	    # get the master title
	    $q="SELECT title FROM things WHERE objid = \"$results->{'ref_from'}\"";
	    $master_row = &query($q, $dbh);

	    if ($master_row->{'numrows'}) {

		$results->{'title'} = $master_row->{'title'};

		# get the body to determine the text to take for th title
		$q="SELECT body FROM extraprop WHERE objid = \"$results->{'axisnames'}\" and fieldname = 'encodings'";
		if ($debug) {
		    print VAR_ERLOG "Extraprop query --> $q\n";
		}

		$extraprop = &query($q, $dbh);
		if ($extraprop->{'numrows'}) {
		    if ($debug) {
			print VAR_ERLOG "Body is $extraprop->{'body'}\n";
		       }
		    @rows = split (/\n/,$extraprop->{'body'});
		    foreach $row (@rows) {

			# get rid of CR!
			$row =~ s/\r//g;
                        # NEED TO WORK ON FOR MULTI DIMENSIONAL VARIANCE
			($code, $val) = split(/-/,$row);       
			# look for the vcode in the enumerate list
			if ($code eq $results->{'vcode'}) {
			    $results->{'title'} .= " - " . $val;
                            if ($debug) {
				print VAR_ERLOG "Title is .... $results->{'title'} for variant!\n"
                            }
                            break;
			}
		    }
		}
	    }
	}  # end of populate title for variant
	$results->{'title'} = string_exdbi( $results->{'title'} );
    }
    return ($results);
}


# Pass in the master object ID, data base handle, and a pointer to an array (\@variants), and
# return all of the object ID's in $variants[$i][0], and object titles in $variants[$i][1] for the variants

sub getVariants {
    my ($master_objid, $dbh, $variants) = @_;
   
    my ($q, $sth);

    $q = "SELECT objid FROM things WHERE ref_from = \"$master_objid\" and objid <> \"$master_objid\"";
    if ($debug) {
	print VAR_ERLOG "getVariants:  query --> $q\n";
    }

    $sth = $dbh->prepare($q);
    $sth->execute;
    $rows = $sth->rows;
    for ($i = 0; $i < $rows; $i++) {
	$res = 	$sth->fetchrow_hashref;
	$$variants[$i][0] = $res->{'objid'};
	$titles = &get_fields_and_title("","",$$variants[$i][0], $dbh);
	$$variants[$i][1] = $titles->{'title'};

    }
	
   if ($debug) {
	print VAR_ERLOG "getVariants: Number of rows are $rows\n";
	for ($i =0; $i <= $rows; $i++) {
	    print VAR_ERLOG "getVariants: Variants are $$variants[$i][0] --> $$variants[$i][1]\n";
	}
    }

    return $rows;
}

#  Pass in an array of arrays [[objid][objtitle],[][],], of variant information
#  Create pulldown
sub gen_variant_sel {
    my ($base, $l, $numVar, @variants) = @_;
    my ($x, $v, $d);

    $x="";

    $d=1;
    for ($i = 0; $i < $numVar; $i++) {
	$x.= "<OPTION VALUE=$variants[$i][0] ";  # GF removed quotes
	if (( $base ) and ( $d ) and ( length($base)>=$l )) {
	    if ( "$base" eq "$k" ) {
		$x.="SELECTED $k = $base";
		$d=0;
	    }
	}
	$x.="> $variants[$i][1] \n";
	if ($debug) {
	    print VAR_ERLOG "gen_variant_sel: $x\n";
	}
   }
   return $x;
}

#
# To submit a query and return the result.  Needs to be expanded to return more than 1 line.
#
sub query {
    my ($q, $dbh) = @_;

    my ($sth, $rows, $results);

    $sth=$dbh->prepare($q);
    $sth->execute;
    $rows = $sth->rows;
    if ($rows <= 0) {
	$sth->finish;
	return undef;
    }

    # get the row if it is a select statement, otherwise we do not need the results
    if ($q =~ /SELECT/) {
	$results = $sth->fetchrow_hashref;
	$results->{'numrows'} = $rows;
	return $results;
    }
}

1;


