#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#  variant.pm
#
#   NOTE:  $relpath MUST BE SET BY THE SCRIPT THAT IS REQUIRING THIS LIBRARY
#
# Version 1.0.3 IB 20000608
#    removed -w from the first line
#    commented out the statement: $debug = 1;
# Version 1.0.2 TG 20000602
#    added multi-variant support
# Version 1.0.1 TG 20000601
#    added getVariant
# Version 1.0.0 TG 20000531 
#    created the first version

require "$relpath/lib/formtools.pm";

# $debug = 1;
if ($debug) {
    open VAR_ERLOG, ">/tmp/variant";
}

###########################################################################
# PURPOSE     
#   To retreive the title for objects, as well as any additional requested 
#   fields.  A simple query to the things table is only alright if the item
#   is not a variant.  If it is a variant, it gets the title based on the 
#   master title and the body of the extraprop table for the ref_from objid
#
# INPUT  
#   $fields - additional fields to include in the selection from things
#   $where_clause - additional statements to be added to the whereclause 
#        for the query from things
#   $objid - the primary key of things, to base the query on
#   $dbh - current active database handler
#
# OUTPUT
#   $results - a hash, to be referenced by the fields queried in things
#
# NOTES
#   Only 1 row will ever be returned, as objid is the primary key of things.
############################################################################
sub get_fields_and_title {
    my ($fields) = $_[0];
    my ($where_clause) = $_[1];
    my ($objid) = $_[2];
    my ($dbh) = $_[3];

    my ($q, $sth, @results_arr, @master_arr, @extraprop_arr, $results);
    my ($extraprop, $master_row, @rows, $row, $title);
    my ($multiple_var, $vcode_indx, @vcodes, @variants);

    # Added axisnames, ref_from and vcode to populate title of variant
    $q="SELECT title, axisnames, ref_from, vcode";

    # if there are additional fields append them
    if ($fields) {
	$q .= ", $fields";
    }

    # Since objid is the primary key, only one row will be returned from 
    # this query
    $q .= " from things where objid='$objid'";

    # if there are additional where clause cases, append them
    if ($where_clause) {
	$q .= " $where_clause";
    }

    if ($debug) {
	print VAR_ERLOG "Select from things with --> $q\n";
    } 

    $numRows = &query($q, $dbh, \@results_arr);

    # we know that there is only one row returned, so get it!
    $results = $results_arr[0];
    $results->{'numrows'} = $numRows;
       
    if ( $numRows) {
	if ($results->{'ref_from'} ne $objid && $results->{'ref_from'} ne "") {
	    if ($debug) {
		print VAR_ERLOG "Need to set objid variant title!\n";
	    }
	    
	    # get the master title
	    $q="SELECT title FROM things WHERE objid = \"$results->{'ref_from'}\"";
	    $numRows = &query($q, $dbh, \@master_arr);
	    
	    # only will have one returned result
	    $master_row = $master_arr[0];
	    
	    if ($numRows) {
		$results->{'title'} = $master_row->{'title'};
		$multiple_var = $results->{'axisnames'};
		if ($multiple_var =~ /:/) {
		    if ($debug) {
			print VAR_ERLOG "$results->{'objid'} has multiple variants! $results->{'ref_from'}\n";
		    }
		    @variants = split /:/, $multiple_var;
		    @vcodes = split /:/, $results->{'vcode'};
		}
		else {
		    $variants[0] = $results->{'axisnames'};
		    $vcodes[0] = $results->{'vcode'};
		}

		# append the title for each variant!
		$vcode_indx = 0;
		foreach $variant (@variants) {
		    
		    # get the body to determine the title
		    $q="SELECT body FROM extraprop WHERE objid = \"$variant\" and fieldname = 'encodings'";
		    if ($debug) {
			print VAR_ERLOG "Extraprop query --> $q\n";
		    }
		    
		    $numRows = &query($q, $dbh, \@extraprop_arr);
		    # only have one result returned
		    
		    $extraprop = $extraprop_arr[0];
		    
		    if ($numRows) {
			if ($debug) {
			    print VAR_ERLOG "Body is $extraprop->{'body'}\n";
			}
			@rows = split (/\n/,$extraprop->{'body'});
			$while_indx = 0;
			$done = 0;
			while ((!$done) && ($while_indx <= $#rows)) {
			    $row = $rows[$while_indx++];
			    # get rid of CR!
			    $row =~ s/\r//g;
			    ($code, $val) = split(/-/,$row);       
			    # look for the vcode in the enumerate list
			    if ($code eq $vcodes[$vcode_indx]) {
				$results->{'title'} .= " - " . $val;
				if ($debug) {
				    print VAR_ERLOG "Title is .... $results->{'title'} for variant!\n"
				    }
				$done = 1;
			    }
			}
		    }
		    $vcode_indx++;
		}
	    }  # end of populate title for variant
	}
	$results->{'title'} = string_exdbi( $results->{'title'} );
    }
    return ($results);
}

###########################################################################
# PURPOSE     
#   To retreive the objid and titles for all variants associated with a 
#   master objid.
#
# INPUT  
#   $master_objid - the master's objid.  The item you want to find the 
#                   variants for
#   $dbh - current active database handler
#   $variants - pointer to an array, used as output.  Must be passed in 
#               as \@arrayName
#
# OUTPUT
#   $variants - a two dimensional array, which will consist of the variant
#               name ($variant[<indx>][0]) and title ($variant[<indx>[1]), 
#               where
#               indx is 0 --> number of variants - 1.
#
# NOTES
#   Only 1 row will ever be returned, as objid is the primary key of things.
###########################################################################
sub getVariants {
    my ($master_objid, $dbh, $variants) = @_;
   
    my ($q, $sth, $res, @res_arr, $numRows);

    $q = "SELECT objid FROM things WHERE ref_from = \"$master_objid\" and objid <> \"$master_objid\"";
    if ($debug) {
	print VAR_ERLOG "getVariants:  query --> $q\n";
    }

    $numRows = &query($q, $dbh, \@res_arr);

    for ($i = 0; $i < $numRows; $i++) {
	$res = $res_arr[$i];
	$$variants[$i][0] = $res->{'objid'};
	$titles = &get_fields_and_title("","",$$variants[$i][0], $dbh);
	$$variants[$i][1] = $titles->{'title'};
    }
	
   if ($debug) {
	print VAR_ERLOG "getVariants: Number of rows are $numRows\n";
	for ($i =0; $i <= $numRows; $i++) {
	    print VAR_ERLOG "getVariants: Variants are $$variants[$i][0] --> $$variants[$i][1]\n";
	}
    }

    return $numRows;
}

###########################################################################
# PURPOSE     
#   To create a dynamic pull down for variants.  The pulldown is based on 
#   the array of variant information passed into this routine.
#
# INPUT  
#   $base - 
#   $numVar - The number of variants to include in the pulldown list
#   @variants - Array of variants, details of datastructure in OUTPUT 
#               section of getVariants procedure above.
#
# OUTPUT
#   $options - The HTML <OPTION ...> statements to be put within a 
#              SELECT HTML statement
#
# NOTES
#
###########################################################################
sub gen_variant_sel {
    my ($base, $numVar, @variants) = @_;
    my ($options, $v, $d);

    $options="";
    $d=1;
    for ($i = 0; $i < $numVar; $i++) {
	$options.= "<OPTION VALUE=$variants[$i][0] ";  
	$options.="> $variants[$i][1] \n";
	if ($debug) {
	    print VAR_ERLOG "gen_variant_sel: $options\n";
	}
   }
   return $options;
}

###########################################################################
# PURPOSE     
#   To submit a query to the $dbh, returning results if there are any
#
# INPUT  
#   $q - query to submit
#   $dbh - active open database handler
#   $p_results - A pointer to an array to return the results back in.
#
# OUTPUT
#   $p_results -  An array of hashes, each index consisting on the simple 
#                 has with the keys being the fields requested in the query
#   $rows - The number of rows returned
#
# NOTES
#
###########################################################################
sub query {
    my ($q, $dbh, $p_results) = @_;

    my ($sth, $rows, $results, $i);

    $sth=$dbh->prepare($q);
    $sth->execute;
    $rows = $sth->rows;
    if ($rows <= 0) {
	$sth->finish;
	return undef;
    }

    # get the row if it is a select statement, otherwise we do not need 
    # the results
    if ($q =~ /SELECT/) {
	# get ALL rows returned
	for ($i = 0; $i < $rows; $i++) {
	    $$p_results[$i] = $sth->fetchrow_hashref;
	}
	return $rows;
    }
}

1;





