#!/usr/bin/perl
#
# initialize_tables
#
# Version 4-20000518 RF
#    real work on table creation and insert statements
# Version 3-20000518 RF
#    more data parsing work, table creation...
# Version 2-20000516 RF
#    table creation and population for MySQL added
#    stub for language-specific messages
# Version 1-20000515 RF
#    created to locate the data sources and then
#       verify connectability, create database, and
#       create and populate the tables.
#    writes directly into the /conf directory, either
#       conf files, or site_configuration.txt
#

# OVERVIEW:
#    This script will attempt to locate connectable databases
# via the Perl DBI, then request some basic configuration 
# information.
#    Run from /tools, and the /bits/{lang}/system directory must exist
# in the expected place, with data load files.
#    If data load files are not found for the requested language, then
# the ones in the "eng" directory will be used, if present.
#


## default parameters
#
$script_version="3 of 20000518";
$script_name="initialize_tables.cgi";

$messages=messages();

## check to see if passed any arguments
#
if ( @ARGV ) {
   $ok=1;
   if ( $ARGV[0] =~ /^(-h|--h)/ ) {         # print help text
      print "$script_name version $script_version useage:\n";
      print "   Copyright 2000 by OpenSales, Inc.\n";
      print "\n";
      print "   $script_name [-h][-v]\n";
      print "\n";
      print "   -h --help, display this message and exit\n";
      print "   -v --version, print version only and exit\n";
      print "\n";
      $ok=0;
   } elsif ( $ARGV[0] =~ /^(-v|--v)/ ) {    # print version
      print "$script_name version $script_version\n";
      $ok=0;
   }                                        # else nothing... 
   exit 1 if ( ! $ok );
} # done checking for command line arguments...

## check to see if we have a /conf/known_resources.txt file
#
@temp=();
if ( -f "../conf/known_resources.txt" ) {
   open( FH, "<../conf/known_resources.txt" );
   @temp = (<FH>);
   close(FH);
   chomp @temp;
} else {
   print "Warning - no 'known_resources.txt' file in /conf!\n";
   print "   you may want to run './check_prerequisites.cgi'!\n";
   print "\n";
} # done checking resources file

## construct the resources hash 
#
%resources = split /\t/, @temp;

use DBI;

## check to see if use DBI failed...
#

# TBD

## look for data drivers
#
$c=0;
@sources=();

print "Starting to examine available Perl::DBI data sources:\n";
@temp = DBI->available_drivers;
foreach $k (@temp) {
   next if ( $k eq "Proxy" );
   print "Driver $k has sources:\n";
   @temp2 = DBI->data_sources($k);

   ## check for an error message raised here...
   #
   if ( $DBI::err ) {
      $l=$DBI::errstr;
      print "ERROR - $l\n";
      @temp2=();
   } # done testing for DBI error

   ## print out the data sources...
   #
   foreach (@temp2) {
      print "     $_\n";
      push @sources, ( "($c)\t$k\t$_" );
      $c++;
   }
   print "\n";
} # done running through the available data drivers

## print menu of available data sources
#
print "\n\n\n\n\n";
print "Please select the data source you wish to use:\n";

foreach $k ( @sources ) { print "$k\n"; }

$ok=0;
while ( ! $ok ) {
   $k=<STDIN>;
   $ok=1 if ( ( 0<=$k ) and ( $c > $k ));
   $k=$c-1;
   print "Allowed choices are 0 through $k!\n" if ( ! $ok );
} # done getting a data source from list
$site->{'datasource'}=$k;
print "\n\n\n\n\n";

## extract the table name and set driver specific parameters
#
($datadriver,$datasource)=( split /\t/, $site->{'datasource'} )[1..2];
if ( $datadriver eq "mysql" ) {
   # table name is third block
   $site->{'site_database'}=( split /:/, $datasource )[2];

} elsif ( $datadriver eq "" ) {

} # end of driver specific parameter sets


## ask for the usual parameters
#

@questions=(
   "What is the database server?",
   "What is the default server port?",
   "What is the user name to connect as?",
   "What is the password for this account?",
   "What language initial data load should be used?"
);

@keynames=(
   "site_database_serv",
   "site_database_port",
   "site_dbiuser",
   "site_dbipwd",
   "language"
);

@defaults=(
   "localhost",
   "3306",
   "",
   "",
   "eng"
);

foreach $c ( 0..4 ) {
   print "$questions[$c] [$defaults[$c]] ";
   $k=<STDIN>;
   chomp $k;
   $k=$defaults[$c] if ( ! $k );
   $site->{$keynames[$c]}=$k;
   print "\n";
}
## print a summary of the configuration parameters


## check for initial data load files
#
$class_file="../bits/$site->{'language'}/system/os_classes.txt";
if ( ! -f $class_file ) {
   print "$messages->{'200'}\n";
   $class_file="../bits/eng/system/os_classes.txt"; 
   if ( ! -f $class_file ) {
      print "$messages->{'201'}\n";
      $class_file="";
   }
}

$object_file="../bits/$site->{'language'}/system/os_objects.txt";
if ( ! -f $object_file ) {
   print "$messages->{'202'}\n";
   $object_file="../bits/eng/system/os_objects.txt"; 
   if ( ! -f $object_file ) {
      print "$messages->{'203'}\n";
      $object_file="";
   }
}

## display the targets and/or exit
#
print "Initial Class Set = $class_file \n";
print "Initial Data Set  = $object_file \n\n\n";

if ( ( ! $object_file ) or ( ! $class_file ) ) {
   print "$messages->{'204'}\n";
   exit 1;
} # done 

open( FE, ">bad.txt");

## load initial class set
#
print "Loading Classes...\n";
%classes=();
@fields=();                  # stores current field definition list   
open(FH, "<$class_file");
while ($l=<FH>) {
   chomp $l;
   if ( $l =~ /^#:/ ) {                # new field list
      @fields = split /\"\t\"/, $l;
      $fields[0] =~ s/^#:\"//;         # strip remaining leading quote et al
      $fields[$#fields] =~ s/\"$//;    # strip remaining trailing quote
      print "Field list is now:\n";
      foreach ( @fields ) { print ; print " - "; }
      print "\n\n";
   }                         # done with new field load, if applicable
   next if ( $l =~ /^#/ );   # skip comment lines
   next if ( $l !~ /\t/ );   # all useful lines have tabs...

   ## interpret remaining lines as class records
   #
   @temp = split /\"\t\"/, $l;
   # print "$#fields fields -> $#temp actual \n";
   if ( $#temp != $#fields ) {         # check for attribute count mismatch
      print FE "BAD LINE:$l\n";
      next;
   }
   foreach (@temp) { s/^\"|\"$//; }    # kill leading and trailing quotes
   $classes{$temp[0]}={};    # blank hashref
   foreach (0..$#fields) { $classes{$temp[0]}->{$fields[$_]}=$temp[$_]; } 
}                            # done reading through class list
close(FH);

## load initial data set
#
print "Loading Objects...\n";
%objects=();
@fields=();                  # stores current field definition list   
open(FH, "<$object_file");
while ($l=<FH>) {
   chomp $l;
   if ( $l =~ /^#:/ ) {                # new field list
      @fields = split /\"\t\"/, $l;
      $fields[0] =~ s/^#:\"//;         # strip remaining leading quote et al
      $fields[$#fields] =~ s/\"$//;    # strip remaining trailing quote
      print "Field list is now:\n";
      foreach ( @fields ) { print ; print " - "; }
      print "\n\n";
   }                         # done with new field load, if applicable
   next if ( $l =~ /^#/ );   # skip comment lines
   next if ( $l !~ /\t/ );   # all useful lines have tabs...

   ## interpret remaining lines as objects
   #
   @temp = split /\"\t\"/, $l;
   if ( $#temp != $#fields ) {         # check for attribute count mismatch
      print FE "BAD LINE:$l\n";
      next;
   }
   foreach (@temp) { s/^\"|\"$//; }    # kill leading and trailing quotes
   $objects{$temp[0]}={};              # blank hashref
   foreach (0..$#fields) { $objects{$temp[0]}->{$fields[$_]}=$temp[$_]; } 
}                            # done reading through initial dataset
close(FH);
close(FE);

## locate and prepare table definitions
#
foreach $k ( keys %objects ) {
   next if ( $objects{$k}->{'objclass'} ne "106-A" );
   @temp = split /\\n/, $objects{$k}->{'os_table_defn'};

}

## locate and prepare class->table mappings
#



## language-sensitive messages...
#
sub messages {
   $messages = {

   '100'=>"",
   '200'=>"ERROR - no class load found for the language, attempting english!",
   '201'=>"ERROR - no class load found for english either!",
   '202'=>"ERROR - no object load found for the language, attempting english!",
   '203'=>"ERROR - no object load found for english either!",
   '204'=>"FATAL ERROR - cannot find one/both essential initial data files!",
   }; 

   return $messages;
} #/messages loading




1;
