#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# parse.pl 
#
# Version 1 IB 20000524
#    created first version
#
#    Read database load file system/os_objects.txt, and load database
#    tables things and extraprop.
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

use DBI;

require "$relpath/conf/site.pm";

$site=setsite();

## open database connection
#
$q="$site->{'db_site_config'}";
$dbh=DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});

## multilingual messages
#
$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );

## load messages
#
%script_msg=();
$script=( split /\./, $scriptname )[0];
&message;

$object_file="$relpath/bits/$om_render_lang/system/os_objects.txt";

if ( ! -f $object_file ) {
   print "$script_msg{2}\n";
   $object_file="$relpath/bits/eng/system/os_objects.txt";
   if ( ! -f $object_file ) {
      print "$script_msg{3}\n";
      $object_file="";
   }
}

%conf=();        # null hash for configuration parameters
%database=();    # temporary holder for parameters from database.conf

if ( $site->{'site_datadriver'} eq '' ) { 
   print "$script_msg{1}\n";
   exit 1;
}

%objects=();
@fields=();                  # stores current field definition list

$objidfld = "";
$objclassfld = "";
$nskfld = "";
$briefld = "";

open(FH, "<$object_file");
while ($l=<FH>) {

   chomp $l;

   if ( $l =~ /^#:/ ) {                # new field list
      @fields = split /\"\t\"/, $l;
      $fields[0] =~ s/^#:\"//;         # strip remaining leading quote et al
      $fields[$#fields] =~ s/\"//;    # strip remaining trailing quote
      foreach (@fields) {
         if ( $_ eq "objid" ) { $objidfld = $_; };
         if ( $_ eq "objclass" ) { $objclassfld = $_; };
         if ( $_ eq "nsk_12001_a" ) { $nskfld = $_; };
         if ( $_ eq "brief" ) { $briefld = $_; };
      }
   }                         # done with new field load, if applicable

   next if ( $l =~ /^#/ );   # skip comment lines
   next if ( $l !~ /\t/ );   # all useful lines have tabs...

   if ( $objidfld eq "objid" && $objclassfld eq "objclass" &&
      $nskfld eq "nsk_12001_a" && $briefld == "brief" ) {

      ## interpret remaining lines as objects
      #
   
      @temp = split /\"\t\"/, $l;
      if ( $#temp != $#fields ) {         # check for attribute count mismatch
         print "BAD LINE:$l\n";
         next;
      }
      foreach (@temp) { s/^\"|\"$//; }    # kill leading and trailing quotes
      $objects{$temp[0]}={};              # blank hashref
      foreach (0..$#fields) { 
         $objects{$temp[0]}->{$fields[$_]}=$temp[$_]; 
      }
      $q = "insert into things ( objid, objclass, brief, status ) ";

      $a = $objects{$temp[0]}->{'objid'};
      $b = $objects{$temp[0]}->{'objclass'};
      $c = $objects{$temp[0]}->{'brief'};
      $d = $objects{$temp[0]}->{'status'};
      $e = $objects{$temp[0]}->{'nsk_12001_a'};

      $q .= "values ( '$a', '$b', '$c', '$d' )";

      $dbh->do($q);
   
      $q = "insert into extraprop ( objid, fieldname, body ) ";
      $q .= "values ( '$a', '$e', '$c' )";
   
      $dbh->do($q);
   } 
}                            # done reading through initial dataset
close(FH);

sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below

   if ($om_render_lang eq "eng" ) {
      $script_msg{1}="Your database driver is not specified, so databases could not be updated.";
      $script_msg{2}="ERROR - no object load found for the language, attempting english!";
      $script_msg{3}="ERROR - no object load found for english either!";
   } else {
      open( EM, "<$relpath/bits/$om_render_lang/system/$script.messages");
      while(<EM>) {
         chomp;
         next if /^\#/;
         next if (! $_);
         @temp=split /\t/;
         $script_msg{$temp[0]}=$temp[1];
      }
      close(EM);
   }
}

1;
